@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -382,10 +382,7 @@ void gp_close_printer(const gs_memory_t *mem,
 
 /* ------ File enumeration ------ */
 
-#ifndef file_enum_DEFINED	/* also defined in iodev.h */
-#  define file_enum_DEFINED
 typedef struct file_enum_s file_enum;
-#endif
 
 /*
  * Begin an enumeration.  pat is a C string that may contain *s or ?s.
