@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -21,10 +21,10 @@
 #  define gscie_INCLUDED
 
 #include "std.h"
-#include "gsrefct.h"
 #include "gsstype.h"		/* for extern_st */
 #include "gstypes.h"		/* for gs_range_t */
 #include "gxctable.h"
+#include "gscspace.h"
 
 /* ---------------- Configuration parameters ---------------- */
 
@@ -154,10 +154,7 @@ typedef float cie_cached_value;
 
 /* ---------------- Structures ---------------- */
 
-#ifndef gs_cie_render_DEFINED
-#  define gs_cie_render_DEFINED
 typedef struct gs_cie_render_s gs_cie_render;
-#endif
 
 /* ------ Common definitions ------ */
 
@@ -638,10 +635,7 @@ struct gx_cie_joint_caches_s {
     bool skipEncodeLMN;
 };
 
-#ifndef gx_cie_joint_caches_DEFINED
-#define gx_cie_joint_caches_DEFINED
 typedef struct gx_cie_joint_caches_s gx_cie_joint_caches;
-#endif
 
 #define private_st_joint_caches() /* in gscie.c */\
   gs_private_st_simple(st_joint_caches, gx_cie_joint_caches,\
