@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -373,9 +373,9 @@ gx_concretize_Separation(const gs_client_color *pc, const gs_color_space *pcs,
 }
 
 static int
-gx_remap_concrete_Separation(const frac * pconc,  const gs_color_space * pcs,
+gx_remap_concrete_Separation(const gs_color_space * pcs, const frac * pconc,
         gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-                             gs_color_select_t select)
+                             gs_color_select_t select, const cmm_dev_profile_t *dev_profile)
 {
 #ifdef DEBUG
     /*
@@ -389,7 +389,7 @@ gx_remap_concrete_Separation(const frac * pconc,  const gs_color_space * pcs,
         const gs_color_space *pacs = pcs->base_space;
 
         return (*pacs->type->remap_concrete_color)
-                                (pconc, pacs, pdc, pgs, dev, select);
+                                (pacs, pconc, pdc, pgs, dev, select, dev_profile);
     } else {
         /* We need to determine if we did a direct color replacement */
         gx_remap_concrete_separation(pconc[0], pdc, pgs, dev, select);
