@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -22,6 +22,8 @@
 #include "gsmemory.h"
 #include "gsiparam.h"
 #include "gsrefct.h"
+#include "gsgstate.h"
+#include "gsccolor.h"
 
 /*
  * Previous versions had a complicated lifecycle discipline for
@@ -133,30 +135,10 @@
 */
 
 /* Opaque types for a graphics state stuff */
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-#ifndef gsicc_link_DEFINED
 typedef struct gsicc_link_s gsicc_link_t;
-#  define gsicc_link_DEFINED
-#endif
-
 
 /* Define ICC profile structure type */
-#ifndef cmm_profile_DEFINED
-#define cmm_profile_DEFINED
 typedef struct cmm_profile_s cmm_profile_t;
-#endif
-
-/*
- * Define the abstract type for color space objects.
- */
-#ifndef gs_color_space_DEFINED
-#  define gs_color_space_DEFINED
-typedef struct gs_color_space_s gs_color_space;
-#endif
 
 typedef struct cmm_dev_profile_s cmm_dev_profile_t;
 
@@ -219,15 +201,8 @@ typedef struct gs_cie_abc_s gs_cie_abc;
 typedef struct gs_cie_def_s gs_cie_def;
 typedef struct gs_cie_defg_s gs_cie_defg;
 
-#ifndef gs_device_n_map_DEFINED
-#  define gs_device_n_map_DEFINED
 typedef struct gs_device_n_map_s gs_device_n_map;
-#endif
-
-#ifndef gs_device_n_attributes_DEFINED
-#  define gs_device_n_attributes_DEFINED
 typedef struct gs_device_n_attributes_s gs_device_n_attributes;
-#endif
 
 /*
  * Non-base direct color spaces: Separation and DeviceN.
@@ -350,6 +325,7 @@ gs_color_space *gs_cspace_new_DeviceRGB(gs_memory_t *mem);
 gs_color_space *gs_cspace_new_DeviceCMYK(gs_memory_t *mem);
 gs_color_space *gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs, 
                                   int components);
+gs_color_space *gs_cspace_new_scrgb(gs_memory_t *pmem, gs_gstate * pgs);
 
 /* ------ Accessors ------ */
 
@@ -375,10 +351,6 @@ bool gs_color_space_equal(const gs_color_space *pcs1,
                           const gs_color_space *pcs2);
 
 /* Restrict a color to its legal range. */
-#ifndef gs_client_color_DEFINED
-#  define gs_client_color_DEFINED
-typedef struct gs_client_color_s gs_client_color;
-#endif
 void gs_color_space_restrict_color(gs_client_color *, const gs_color_space *);
 
 /*
