@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -20,6 +20,7 @@
 #  define gsdsrc_INCLUDED
 
 #include "gsstruct.h"
+#include "scommon.h"
 
 /* ---------------- Types and structures ---------------- */
 
@@ -34,11 +35,6 @@
  * at the moment.
  */
 
-#ifndef stream_DEFINED
-#  define stream_DEFINED
-typedef struct stream_s stream;
-#endif
-
 /*
  * Prepare to access a block of data from a source.  buf must be a client-
  * supplied buffer of at least length bytes.  If ptr == 0, always copy the
@@ -56,10 +52,7 @@ typedef enum {
     data_source_type_floats,
     data_source_type_stream
 } gs_data_source_type_t;
-#ifndef gs_data_source_DEFINED
-#  define gs_data_source_DEFINED
 typedef struct gs_data_source_s gs_data_source_t;
-#endif
 struct gs_data_source_s {
     data_source_proc_access((*access));
     gs_data_source_type_t type;
