@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -747,7 +747,7 @@ static void remove_free(gs_memory_chunk_t *cmem, chunk_free_node_t *node)
     remove_free_size(cmem, node);
 }
 
-#ifdef MEMENTO
+#if defined(MEMENTO) || defined(SINGLE_OBJECT_MEMORY_BLOCKS_ONLY)
 #define SINGLE_OBJECT_CHUNK(size) (1)
 #else
 #define SINGLE_OBJECT_CHUNK(size) ((size) > (CHUNK_SIZE>>1))
@@ -1409,7 +1409,7 @@ chunk_object_type(const gs_memory_t * mem, const void *ptr)
 }
 
 static int
-chunk_register_root(gs_memory_t * mem, gs_gc_root_t * rp, gs_ptr_type_t ptype,
+chunk_register_root(gs_memory_t * mem, gs_gc_root_t ** rp, gs_ptr_type_t ptype,
                  void **up, client_name_t cname)
 {
     return 0;
