@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -39,6 +39,7 @@
 #ifndef gsmemory_INCLUDED
 #  define gsmemory_INCLUDED
 
+#include "std.h"
 #include "gstypes.h"		/* for gs_bytestring */
 #include "gslibctx.h"
 
@@ -46,13 +47,6 @@
 typedef struct gs_memory_struct_type_s gs_memory_struct_type_t;
 typedef const gs_memory_struct_type_t *gs_memory_type_ptr_t;
 
-/* Define the opaque type for an allocator. */
-/* (The actual structure is defined later in this file.) */
-#ifndef gs_memory_DEFINED
-#  define gs_memory_DEFINED
-typedef struct gs_memory_s gs_memory_t;
-#endif
-
 /* Define the opaque type for a pointer type. */
 typedef struct gs_ptr_procs_s gs_ptr_procs_t;
 typedef const gs_ptr_procs_t *gs_ptr_type_t;
@@ -347,15 +341,22 @@ typedef struct gs_memory_procs_s {
   (*(mem)->procs.free_string)(mem, data, nbytes, cname)
     gs_memory_proc_free_string((*free_string));
 
-    /*
-     * Register a root for the garbage collector.  root = NULL
-     * asks the memory manager to allocate the root object
-     * itself (immovable, in the manager's parent): this is the usual
-     * way to call this procedure.
-     */
+/*
+  Register a root for the garbage collector.
+    We have three scenarios to deal with:
+    1) rpp is NULL:
+       the root will remain internal to the memory manager.
+    2) rpp is valid, but *rpp is NULL:
+       We'll allocate the root, and return a pointer to it (via rpp).
+    3) rpp is valid, and *rpp is valid:
+       The caller has supplied the root, so use it.
+
+    Scenario 1 is a root that will last until the memory mamanger is
+    shutdown. 2 & 3 allow the root to be unregistered by the caller.
+*/
 
 #define gs_memory_proc_register_root(proc)\
-  int proc(gs_memory_t *mem, gs_gc_root_t *root, gs_ptr_type_t ptype,\
+  int proc(gs_memory_t *mem, gs_gc_root_t **root, gs_ptr_type_t ptype,\
     void **pp, client_name_t cname)
 #define gs_register_root(mem, root, ptype, pp, cname)\
   (*(mem)->procs.register_root)(mem, root, ptype, pp, cname)
@@ -431,7 +432,7 @@ void *gs_resize_struct_array(gs_memory_t *mem, void *obj, uint num_elements,
                              client_name_t cname);
 
 /* Register a structure root.  This just calls gs_register_root. */
-int gs_register_struct_root(gs_memory_t *mem, gs_gc_root_t *root,
+int gs_register_struct_root(gs_memory_t *mem, gs_gc_root_t **root,
                             void **pp, client_name_t cname);
 
 /* Define no-op freeing procedures for use by enable_free. */
