@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -22,6 +22,7 @@
 #include "gscompt.h"
 #include "gsrefct.h"
 #include "gxbitfmt.h"
+#include "gsdevice.h"
 
 /*
  * Because compositor information is passed through the command list,
@@ -51,22 +52,9 @@
 /*
  * Define the abstract superclass for all compositing function types.
  */
-                                                   /*typedef struct gs_composite_s gs_composite_t; *//* in gscompt.h */
+/*typedef struct gs_composite_s gs_composite_t; *//* in gscompt.h */
 
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-#ifndef gx_device_DEFINED
-#  define gx_device_DEFINED
-typedef struct gx_device_s gx_device;
-#endif
-
-#ifndef gx_device_clist_writer_DEFINED
-#define gx_device_clist_writer_DEFINED
 typedef struct gx_device_clist_writer_s gx_device_clist_writer;
-#endif
 
 typedef struct gs_composite_type_procs_s {
 
