@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -107,14 +107,14 @@ gx_color_index
 gx_device_black(gx_device *dev)
 {
     if (dev->cached_colors.black == gx_no_color_index) {
-        const gx_cm_color_map_procs * cm_procs;
+        subclass_color_mappings scm;
         uchar i, ncomps = dev->color_info.num_components;
         frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
 
-        cm_procs = get_color_mapping_procs_subclass(dev);
+        scm = get_color_mapping_procs_subclass(dev);
         /* Get color components for black (gray = 0) */
-        map_gray_subclass(cm_procs, dev, frac_0, cm_comps);
+        map_gray_subclass(scm, frac_0, cm_comps);
 
         for (i = 0; i < ncomps; i++)
             cv[i] = frac2cv(cm_comps[i]);
@@ -127,14 +127,14 @@ gx_color_index
 gx_device_white(gx_device *dev)
 {
     if (dev->cached_colors.white == gx_no_color_index) {
-        const gx_cm_color_map_procs * cm_procs;
+        subclass_color_mappings scm;
         uchar i, ncomps = dev->color_info.num_components;
         frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
 
-        cm_procs = get_color_mapping_procs_subclass(dev);
+        scm = get_color_mapping_procs_subclass(dev);
         /* Get color components for white (gray = 1) */
-        map_gray_subclass(cm_procs, dev, frac_1, cm_comps);
+        map_gray_subclass(scm, frac_1, cm_comps);
 
         for (i = 0; i < ncomps; i++)
             cv[i] = frac2cv(cm_comps[i]);
@@ -847,10 +847,17 @@ gx_dc_pure_fill_masked(const gx_device_color * pdevc, const byte * data,
             scolors[1] = gx_device_white(dev);
         }
         tcolors[0] = tcolors[1] = pdevc->colors.pure;
+
+        if (invert)
+            lop = rop3_invert_S(lop);
+
+        if (!rop3_uses_S(lop))
+            lop |= rop3_S;
+            
         return (*dev_proc(dev, strip_copy_rop))
             (dev, data, data_x, raster, id, scolors,
              NULL, tcolors, x, y, w, h, 0, 0,
-             (invert ? rop3_invert_S(lop) : lop) | (rop3_S | lop_S_transparent));
+             lop | lop_S_transparent);
     }
 }
 
