@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -17,29 +17,13 @@
 #define gxfapi_INCLUDED
 
 #include "gsmemory.h"
-#include "gsmatrix.h"
 #include "gsccode.h"
 #include "stdint_.h"
+#include "gsgstate.h"
+#include "gsfont.h"
+#include "gstext.h"
 
-#ifndef gs_font_base_DEFINED
-#  define gs_font_base_DEFINED
 typedef struct gs_font_base_s gs_font_base;
-#endif
-
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-#ifndef gs_text_enum_DEFINED
-#  define gs_text_enum_DEFINED
-typedef struct gs_text_enum_s gs_text_enum_t;
-#endif
-
-#ifndef gs_font_DEFINED
-#  define gs_font_DEFINED
-typedef struct gs_font_s gs_font;
-#endif
 
 typedef int fracint;            /* A fractional integer with statically unknown number of fraction bits.
                                    The number of bits depends on plugin and is being specified in
@@ -289,11 +273,6 @@ typedef struct
     int black_width, black_height;
 } gs_fapi_raster;
 
-#ifndef gs_fapi_server_DEFINED
-#define gs_fapi_server_DEFINED
-typedef struct gs_fapi_server_s gs_fapi_server;
-#endif
-
 typedef enum gs_fapi_descendant_code_s
 {                               /* Possible values are descendant font indices and 4 ones defined below. */
     gs_fapi_descendant_prepared = -1,   /* See FAPI_prepare_font in zfapi.c . */
