@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -59,8 +59,8 @@ iclass_proc(gs_image_class_3_mono);
 static irender_proc(image_render_mono);
 static irender_proc(image_render_mono_ht);
 
-irender_proc_t
-gs_image_class_3_mono(gx_image_enum * penum)
+int
+gs_image_class_3_mono(gx_image_enum * penum, irender_proc_t *render_fn)
 {
 #if USE_FAST_HT_CODE
     bool use_fast_code = true;
@@ -111,6 +111,9 @@ gs_image_class_3_mono(gx_image_enum * penum)
                 }
             }
             code = dev_proc(penum->dev, get_profile)(penum->dev, &dev_profile);
+            if (code < 0)
+                return code;
+
             /* Define the rendering intents */
             rendering_params.black_point_comp = penum->pgs->blackptcomp;
             rendering_params.graphics_type_tag = GS_IMAGE_TAG;
@@ -167,8 +170,10 @@ gs_image_class_3_mono(gx_image_enum * penum)
             code = image_init_color_cache(penum, penum->bps, penum->spp);
             if (code >= 0) {
                 code = gxht_thresh_image_init(penum);
-                if (code >= 0)
-                    return &image_render_mono_ht;
+                if (code >= 0) {
+                    *render_fn = &image_render_mono_ht;
+                    return code;
+                }
             }
         }
 not_fast_halftoning:
@@ -201,7 +206,7 @@ not_fast_halftoning:
         /* Reset the clues here, rather than in image_render_mono as
          * previously. Even doing so this often may be overzealous. */
         image_init_clues(penum, penum->bps, penum->spp);
-        return &image_render_mono;
+        *render_fn = &image_render_mono;
     }
     return 0;
 }
@@ -522,45 +527,131 @@ image_render_mono(gx_image_enum * penum, const byte * buffer, int data_x,
 
             }
 
-        } else if (penum->posture == image_portrait ||
-                   penum->posture == image_landscape
-            ) {
-
+        } else if (penum->posture == image_portrait) {
             /**************************************
-             * Slow case, not masked, orthogonal. *
+             * Slow case, not masked, portrait. *
              **************************************/
+            dev_proc_fill_rectangle((*fill_proc)) =
+                dev_proc(dev, fill_rectangle);
+            int iy = fixed2int_pixround(yrun);
+            int ih = fixed2int_pixround(yrun + pdyy) - iy;
+            if (ih < 0)
+                iy += ih, ih = -ih;
 
             /* In this case, we can fill runs quickly. */
             /****** DOESN'T DO ADJUSTMENT ******/
             if (stop <= psrc)
                 goto last;
             for (;;) {
-                if (*psrc != run) {
-                    if (run != htrun) {
-                        htrun = run;
+                byte c = *psrc++;
+                if (c != run) {
+                    int ix = fixed2int_pixround(xrun);
+                    int iw = fixed2int_pixround(xl) - ix;
+                    if (iw < 0)
+                        ix += iw, iw = -iw;
+                    switch (run)
+                    {
+                    case 0:
+                        if (!color_is_pure(penum->icolor0))
+                            goto ht_port;
+                        code = (*fill_proc) (dev, ix, iy, iw, ih,
+                                             penum->icolor0->colors.pure);
+                        break;
+                    case 0xff:
+                        if (!color_is_pure(penum->icolor1))
+                            goto ht_port;
+                        code = (*fill_proc) (dev, ix, iy, iw, ih,
+                                             penum->icolor1->colors.pure);
+                        break;
+                    default:
+                        ht_port:
+                        if (run != htrun) {
+                            htrun = run;
 #if USE_SET_GRAY_FUNCTION
-                        code = image_set_gray(run,masked,mask_base,mask_limit,&pdevc,
-                            &cc,pcs,pgs,dev,gs_color_select_source,penum);
-                        if (code < 0)
-                            goto err;
+                            code = image_set_gray(run,masked,mask_base,mask_limit,&pdevc,
+                                    &cc,pcs,pgs,dev,gs_color_select_source,penum);
+                            if (code < 0)
+                                goto err;
 #else
-                        IMAGE_SET_GRAY(run);
+                            IMAGE_SET_GRAY(run);
 #endif
+                        }
+                        code = gx_fill_rectangle_device_rop(ix, iy, iw, ih,
+                                                            pdevc, dev, lop);
+                    }
+                    if (code < 0)
+                        goto err;
+                    xrun = xl;
+                    rsrc = psrc;
+                    if (psrc > stop)
+                        break;
+                    run = c;
+                }
+                dda_next(next.x);
+                if (psrc >= endp)
+                    break;
+            }
+        } else if (penum->posture == image_landscape) {
+
+            /**************************************
+             * Slow case, not masked, landscape. *
+             **************************************/
+            dev_proc_fill_rectangle((*fill_proc)) =
+                dev_proc(dev, fill_rectangle);
+            int ix = fixed2int_pixround(xrun);
+            int iw = fixed2int_pixround(xrun + pdyx) - ix;
+            if (iw < 0)
+                ix += iw, iw = -iw;
+
+            /* In this case, we can fill runs quickly. */
+            /****** DOESN'T DO ADJUSTMENT ******/
+            if (stop <= psrc)
+                goto last;
+            for (;;) {
+                byte c = *psrc++;
+                if (c != run) {
+                    int iy = fixed2int_pixround(yrun);
+                    int ih = fixed2int_pixround(ytf) - iy;
+                    if (ih < 0)
+                        iy += ih, ih = -ih;
+                    switch (run)
+                    {
+                    case 0:
+                        if (!color_is_pure(penum->icolor0))
+                            goto ht_land;
+                        code = (*fill_proc) (dev, ix, iy, iw, ih,
+                                             penum->icolor0->colors.pure);
+                        break;
+                    case 0xff:
+                        if (!color_is_pure(penum->icolor1))
+                            goto ht_land;
+                        code = (*fill_proc) (dev, ix, iy, iw, ih,
+                                             penum->icolor1->colors.pure);
+                        break;
+                    default:
+                        ht_land:
+                        if (run != htrun) {
+                            htrun = run;
+#if USE_SET_GRAY_FUNCTION
+                            code = image_set_gray(run,masked,mask_base,mask_limit,&pdevc,
+                                    &cc,pcs,pgs,dev,gs_color_select_source,penum);
+                            if (code < 0)
+                                goto err;
+#else
+                            IMAGE_SET_GRAY(run);
+#endif
+                        }
+                        code = gx_fill_rectangle_device_rop(ix, iy, iw, ih,
+                                                            pdevc, dev, lop);
                     }
-                    code = (*fill_pgram)(dev, xrun, yrun, xl - xrun,
-                                         ytf - yrun, pdyx, pdyy,
-                                         pdevc, lop);
                     if (code < 0)
                         goto err;
                     yrun = ytf;
-                    xrun = xl;
                     rsrc = psrc;
-                    if (psrc >= stop)
+                    if (psrc > stop)
                         break;
-                    run = *psrc;
+                    run = c;
                 }
-                psrc++;
-                dda_next(next.x);
                 dda_next(next.y);
                 if (psrc >= endp)
                     break;
@@ -811,7 +902,7 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
     int position, k, j;
     int offset_bits = penum->ht_offset_bits;
     int contone_stride = 0;  /* Not used in landscape case */
-    fixed scale_factor, offset;
+    fixed offset;
     int src_size;
     bool flush_buff = false;
     int offset_contone[GX_DEVICE_COLOR_MAX_COMPONENTS];    /* to ensure 128 bit boundary */
@@ -860,7 +951,6 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
             }
             data_length = dest_width;
             dest_height = fixed2int_var_rounded(any_abs(penum->y_extent.y));
-            scale_factor = float2fixed_rounded((float) src_size / (float) dest_width);
 #ifdef DEBUG
             /* Help in spotting problems */
             memset(penum->ht_buffer, 0x00, penum->ht_stride * vdi * spp_out);
@@ -879,7 +969,6 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
             xrun = dda_current(dda_ht);            /* really yrun, but just used here for landscape */
             dest_height = gxht_dda_length(&dda_ht, src_size);
             data_length = dest_height;
-            scale_factor = float2fixed_rounded((float) src_size / (float) dest_height);
             offset_threshold = (-(long)(penum->thresh_buffer)) & 15;
             for (k = 0; k < spp_out; k ++) {
                 offset_contone[k] = (- ((long)(penum->line) +
@@ -1005,9 +1094,9 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
                             devc_contone_gray[position] = psrc[k];
                             position += LAND_BITS;
                         }
-                    } else if (scale_factor == fixed_half) {
+                    } else if (src_size*2 == dest_height) {
                         for (k = 0; k < data_length; k+=2) {
-                            offset = fixed2int_rounded(scale_factor * k);
+                            offset = fixed2int_rounded(fixed_half * k);
                             devc_contone_gray[position] =
                                 devc_contone_gray[position + LAND_BITS] = psrc[offset];
                             position += LAND_BITS*2;
@@ -1135,8 +1224,8 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
                                 dev_value = &(color_cache[*psrc * spp_out]);
                                 for (j = 0; j < spp_out; j++) {
                                     *(devc_contone[j] + position) = dev_value[j];
-                                    position -= LAND_BITS;
                                 }
+                                position -= LAND_BITS;
                                 xr--;
                             }           /* at loop exit xn will be <= xr */
                             psrc++;
@@ -1172,8 +1261,8 @@ image_render_mono_ht(gx_image_enum * penum_orig, const byte * buffer, int data_x
                                 dev_value = &(color_cache[*psrc * spp_out]);
                                 for (j = 0; j < spp_out; j++) {
                                     *(devc_contone[j] + position) = dev_value[j];
-                                    devc_contone[j] += LAND_BITS;
                                 }
+                                position += LAND_BITS;
                                 xr++;
                             }           /* at loop exit xn will be >= xr */
                             psrc++;
