@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -175,7 +175,9 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
 
         bbox.p.x = x0, bbox.p.y = y0;
         bbox.q.x = x1, bbox.q.y = y1;
-        gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
+        code = gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
+        if (code < 0)
+            return code;
         if_debug10m('T', mem,
           "[T]x,y=(%d,%d) w,h=(%d,%d) => (%g,%g),(%g,%g), offset=(%g,%g)\n",
                     x0, y0, w0, h0,
@@ -256,7 +258,7 @@ tile_colored_fill(const tile_fill_state_t * ptfs,
     bool full_transfer = (w == ptfs->w0 && h == ptfs->h0);
     int code = 0;
 
-    if (source == NULL && lop_no_S_is_T(lop) && dev->procs.copy_planes != NULL &&
+    if (source == NULL && lop_no_S_is_T(lop) && dev_proc(dev, copy_planes) != gx_default_copy_planes &&
         ptfs->num_planes > 0) {
             code = (*dev_proc(ptfs->pcdev, copy_planes))
                     (ptfs->pcdev, data + bits->raster * yoff, xoff,
@@ -616,6 +618,7 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
 #ifdef DEBUG
     const gs_memory_t *mem = ptile->ttrans->mem;
 #endif
+    int code;
 
     ptfs->x0 = x0, ptfs->w0 = w0;
     ptfs->y0 = y0, ptfs->h0 = h0;
@@ -631,7 +634,9 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
 
         bbox.p.x = x0, bbox.p.y = y0;
         bbox.q.x = x1, bbox.q.y = y1;
-        gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
+        code = gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
+        if (code < 0)
+            return code;
         if_debug10m('T', mem,
           "[T]x,y=(%d,%d) w,h=(%d,%d) => (%g,%g),(%g,%g), offset=(%g,%g)\n",
                     x0, y0, w0, h0,
@@ -690,9 +695,12 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
                 ptfs->xoff = xoff;
                 ptfs->yoff = yoff;
 
-                /* We only go through blending during tiling, if
-                   there was overlap as defined by the step matrix
-                   and the bounding box */
+                /* We only go through blending during tiling, if there was overlap
+                   as defined by the step matrix and the bounding box.
+                   Ignore if the area is outside the fill_trans_buffer.rect (bug 700719) */
+                if (x > fill_trans_buffer->rect.q.x || x+w < 0 ||
+                    y > fill_trans_buffer->rect.q.y || y+h < 0)
+                    continue;	/* skip the fill (can breakpoint here) */
                 ptile->ttrans->pat_trans_fill(x, y, x+w, y+h, px, py, ptile,
                     fill_trans_buffer);
             }
@@ -724,17 +732,18 @@ tile_rect_trans_simple(int xmin, int ymin, int xmax, int ymax,
     int tile_width  = ptile->ttrans->width;
     int tile_height = ptile->ttrans->height;
     int src_planes = fill_trans_buffer->n_chan + (fill_trans_buffer->has_tags ? 1 : 0);
+    pdf14_buf *buf = fill_trans_buffer->buf;
 
     /* Update the bbox in the topmost stack entry to reflect the fact that we
      * have drawn into it. FIXME: This makes the groups too large! */
-    if (fill_trans_buffer->dirty->p.x > xmin)
-        fill_trans_buffer->dirty->p.x = xmin;
-    if (fill_trans_buffer->dirty->p.y > ymin)
-        fill_trans_buffer->dirty->p.y = ymin;
-    if (fill_trans_buffer->dirty->q.x < xmax)
-        fill_trans_buffer->dirty->q.x = xmax;
-    if (fill_trans_buffer->dirty->q.y < ymax)
-        fill_trans_buffer->dirty->q.y = ymax;
+    if (buf->dirty.p.x > xmin)
+        buf->dirty.p.x = xmin;
+    if (buf->dirty.p.y > ymin)
+        buf->dirty.p.y = ymin;
+    if (buf->dirty.q.x < xmax)
+        buf->dirty.q.x = xmax;
+    if (buf->dirty.q.y < ymax)
+        buf->dirty.q.y = ymax;
     buff_out_y_offset = ymin - fill_trans_buffer->rect.p.y;
     buff_out_x_offset = xmin - fill_trans_buffer->rect.p.x;
 
@@ -851,20 +860,21 @@ tile_rect_trans_blend(int xmin, int ymin, int xmax, int ymax,
     int num_chan    = ptile->ttrans->n_chan;  /* Includes alpha */
     int tag_offset = fill_trans_buffer->n_chan + (fill_trans_buffer->has_shape ? 1 : 0);
     pdf14_device *p14dev = (pdf14_device *) fill_trans_buffer->pdev14;
+    pdf14_buf *buf = fill_trans_buffer->buf;
 
     if (fill_trans_buffer->has_tags == 0)
         tag_offset = 0;
 
     /* Update the bbox in the topmost stack entry to reflect the fact that we
      * have drawn into it. FIXME: This makes the groups too large! */
-    if (fill_trans_buffer->dirty->p.x > xmin)
-        fill_trans_buffer->dirty->p.x = xmin;
-    if (fill_trans_buffer->dirty->p.y > ymin)
-        fill_trans_buffer->dirty->p.y = ymin;
-    if (fill_trans_buffer->dirty->q.x < xmax)
-        fill_trans_buffer->dirty->q.x = xmax;
-    if (fill_trans_buffer->dirty->q.y < ymax)
-        fill_trans_buffer->dirty->q.y = ymax;
+    if (buf->dirty.p.x > xmin)
+        buf->dirty.p.x = xmin;
+    if (buf->dirty.p.y > ymin)
+        buf->dirty.p.y = ymin;
+    if (buf->dirty.q.x < xmax)
+        buf->dirty.q.x = xmax;
+    if (buf->dirty.q.y < ymax)
+        buf->dirty.q.y = ymax;
     buff_out_y_offset = ymin - fill_trans_buffer->rect.p.y;
     buff_out_x_offset = xmin - fill_trans_buffer->rect.p.x;
 
@@ -918,7 +928,7 @@ tile_rect_trans_blend(int xmin, int ymin, int xmax, int ymax,
 
             /* Blend */
             art_pdf_composite_pixel_alpha_8(dst, src, ptile->ttrans->n_chan-1,
-                                            ptile->blending_mode,
+                                            ptile->blending_mode, ptile->ttrans->n_chan-1,
                                             ptile->ttrans->blending_procs, p14dev);
 
             /* Store the color values */
