@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -93,6 +93,27 @@
 #  define volatile		/* */
 #endif
 
+/* restrict is standard in C99, but not in all C++ compilers. */
+#if defined(__STDC_VERSION__) && __STDC_VERSION__ == 199901L /* C99 */
+# if defined(HAVE_RESTRICT) && HAVE_RESTRICT==1
+#  define gs_restrict restrict
+# else /* defined(HAVE_RESTRICT) && HAVE_RESTRICT==1 */
+#  define gs_restrict
+# endif /* defined(HAVE_RESTRICT) && HAVE_RESTRICT==1 */
+#elif defined(_MSC_VER) && _MSC_VER >= 1500 /* MSVC 9 or newer */
+# define gs_restrict __restrict
+#elif __GNUC__ >= 3 /* GCC 3 or newer */
+# if defined(HAVE_RESTRICT) && HAVE_RESTRICT==1
+#  define gs_restrict __restrict
+# else /* defined(HAVE_RESTRICT) && HAVE_RESTRICT==1 */
+#  define gs_restrict
+# endif /* defined(HAVE_RESTRICT) && HAVE_RESTRICT==1 */
+#else /* Unknown or ancient */
+# define gs_restrict
+#endif
+
+
+
 /* Disable 'inline' if the compiler can't handle it. */
 #ifdef __DECC
 #  undef inline
@@ -109,6 +130,15 @@
 #  endif
 #endif
 
+/* Define ourselves a 'forceinline' we can use to more forcefully
+ * tell the compiler to inline something. On all but MSVC this can
+ * drop back to inline. */
+#ifdef _MSC_VER
+#define forceinline __forceinline
+#else
+#define forceinline inline
+#endif
+
 /*
  * Provide a way to include inline procedures in header files, regardless of
  * whether the compiler (A) doesn't support inline at all, (B) supports it
