@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -21,26 +21,13 @@
 
 /* For memento */
 #include "malloc_.h"
+#include "gxfcache.h"
 
-#ifndef TFace_defined
-#define TFace_defined
 typedef struct _TFace TFace;
-#endif
 
-#ifndef TInstance_defined
-#define TInstance_defined
 typedef struct _TInstance TInstance;
-#endif
 
-#ifndef TExecution_Context_defined
-#define TExecution_Context_defined
 typedef struct _TExecution_Context TExecution_Context;
-#endif
-
-#ifndef ttfInterpreter_DEFINED
-#  define ttfInterpreter_DEFINED
-typedef struct ttfInterpreter_s ttfInterpreter;
-#endif
 
 /* Define auxiliary data types for the TT interpreter. */
 
@@ -123,10 +110,6 @@ typedef struct {
 /* Define a capsule for a TT face.
    Diue to historical reason the name is some misleading.
    It should be ttfFace. */
-#ifndef ttfFont_DEFINED
-#  define ttfFont_DEFINED
-typedef struct ttfFont_s ttfFont;
-#endif
 struct ttfFont_s {
     ttfPtrElem t_cvt_;
     ttfPtrElem t_fpgm;
