@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* PLANar devices */
@@ -32,6 +32,7 @@
  *  planm   1 bit Monochrome
  *  planc  32 bit CMYK (8 bits per channel)
  *  plank   4 bit CMYK (1 bit per channel)
+ *  planr   3 bit RGB (1 bit per channel)
  */
 
 /* Define DEBUG_PRINT to enable some debugging printfs. */
@@ -72,6 +73,7 @@ static int planm_print_page(gx_device_printer * pdev, FILE * pstream);
 static int plang_print_page(gx_device_printer * pdev, FILE * pstream);
 static int planc_print_page(gx_device_printer * pdev, FILE * pstream);
 static int plank_print_page(gx_device_printer * pdev, FILE * pstream);
+static int planr_print_page(gx_device_printer * pdev, FILE * pstream);
 
 /* The device procedures */
 
@@ -156,6 +158,8 @@ static const gx_device_procs planc_procs =
   pgpm_procs(planc_map_color_rgb, planc_encode_color, planc_decode_color);
 static const gx_device_procs plank_procs =
   pgpm_procs(planc_map_color_rgb, planc_encode_color, planc_decode_color);
+static const gx_device_procs planr_procs =
+  pgpm_procs(plan_decode_color, gx_default_rgb_map_rgb_color, plan_decode_color);
 
 /* Macro for generating device descriptors. */
 #define plan_prn_device(procs, dev_name, num_comp, depth, max_gray, max_rgb, print_page) \
@@ -177,6 +181,8 @@ const gx_device_printer gs_plank_device =
   plan_prn_device(plank_procs, "plank", 4, 4, 1, 1, plank_print_page);
 const gx_device_printer gs_planc_device =
   plan_prn_device(planc_procs, "planc", 4, 32, 255, 255, planc_print_page);
+const gx_device_printer gs_planr_device =
+  plan_prn_device(planr_procs, "planr", 3, 3, 1, 1, planr_print_page);
 
 /* ------ Initialization ------ */
 
@@ -262,6 +268,29 @@ static void dump_row_pgm(int w, byte **data, FILE *dump_file)
     }
 }
 
+static void dump_row_pnmr(int w, byte **data, FILE *dump_file)
+{
+    byte *r = data[0];
+    byte *g = data[1];
+    byte *b = data[2];
+
+    if (dump_file == NULL)
+        return;
+    while (w) {
+        byte R = *r++;
+        byte G = *g++;
+        byte B = *b++;
+        int s;
+        for (s=7; s>=0; s--) {
+            fputc(255*((R>>s)&1), dump_file);
+            fputc(255*((G>>s)&1), dump_file);
+            fputc(255*((B>>s)&1), dump_file);
+            w--;
+            if (w == 0) break;
+        }
+    }
+}
+
 typedef void (*dump_row)(int w, byte **planes, FILE *file);
 
 static dump_row dump_start(int w, int h, int num_comps, int log2bits,
@@ -278,13 +307,19 @@ static dump_row dump_start(int w, int h, int num_comps, int log2bits,
         row_proc = dump_row_pnmk;
     } else if ((num_comps == 4) && (log2bits == 3)) {
         row_proc = dump_row_pnmc;
+    } else if ((num_comps == 3) && (log2bits == 0)) {
+        row_proc = dump_row_pnmr;
     } else
         return NULL;
     if (dump_file == NULL)
         return row_proc;
-    if (num_comps == 3)
-        fprintf(dump_file, "P6 %d %d 255\n", w, h);
-    else if (num_comps == 4) {
+    if (num_comps == 3) {
+        if (log2bits == 0)
+            fprintf(dump_file, "P7\nWIDTH %d\nHEIGHT %d\nDEPTH 3\n"
+                    "MAXVAL 255\nTUPLTYPE RGB\n# Image generated by %s\nENDHDR\n", w, h, gs_product);
+        else
+            fprintf(dump_file, "P6 %d %d 255\n", w, h);
+    } else if (num_comps == 4) {
         if (log2bits == 0)
             fprintf(dump_file, "P7\nWIDTH %d\nHEIGHT %d\nDEPTH 4\n"
                     "MAXVAL 255\nTUPLTYPE CMYK\n# Image generated by %s\nENDHDR\n", w, h, gs_product);
@@ -547,3 +582,13 @@ planc_print_page(gx_device_printer * pdev, FILE * pstream)
 #endif
     return plan_print_page_loop(pdev, 3, 4, pstream);
 }
+
+/* Print a color-mapped page (rgb, 1 bpc). */
+static int
+planr_print_page(gx_device_printer * pdev, FILE * pstream)
+{
+#ifdef DEBUG_PRINT
+    emprintf(pdev->memory, "planr_print_page\n");
+#endif
+    return plan_print_page_loop(pdev, 0, 3, pstream);
+}
