@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* PLanar Interlaced Banded device */
@@ -655,8 +655,8 @@ plib_create_buf_device(gx_device **pbdev, gx_device *target, int y,
                                           mem, color_usage);
     if (code < 0)
         return code;
-    if ((*pbdev)->procs.get_bits_rectangle == mem_get_bits_rectangle)
-        (*pbdev)->procs.get_bits_rectangle = plib_get_bits_rectangle_mem;
+    if (dev_proc((*pbdev), get_bits_rectangle) == mem_get_bits_rectangle)
+        set_dev_proc((*pbdev), get_bits_rectangle, plib_get_bits_rectangle_mem);
     return 0;
 }
 
