@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -555,7 +555,7 @@ pdf_compute_font_descriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     if (!(desc.Flags & FONT_IS_SYMBOLIC)) {
         desc.Flags |= FONT_IS_ADOBE_ROMAN; /* required if not symbolic */
         desc.XHeight = (int)x_height;
-        if (!small_present && (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType))
+        if (!small_present && (pdev->PDFA == 0 || bfont->FontType != ft_TrueType))
             desc.Flags |= FONT_IS_ALL_CAPS;
         desc.CapHeight = cap_height;
         /*
@@ -606,10 +606,10 @@ pdf_compute_font_descriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     desc.Descent = desc.FontBBox.p.y;
     if (!(desc.Flags & (FONT_IS_SYMBOLIC | FONT_IS_ALL_CAPS)) &&
         (small_descent > desc.Descent / 3 || desc.XHeight > small_height * 0.9) &&
-        (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType)
+        (pdev->PDFA == 0 || bfont->FontType != ft_TrueType)
         )
         desc.Flags |= FONT_IS_SMALL_CAPS;
-    if (fixed_width > 0 && (!pdev->PDFA != 0 || bfont->FontType != ft_TrueType)) {
+    if (fixed_width > 0 && (pdev->PDFA == 0 || bfont->FontType != ft_TrueType)) {
         desc.Flags |= FONT_IS_FIXED_WIDTH;
         desc.AvgWidth = desc.MaxWidth = desc.MissingWidth = fixed_width;
     }
