@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2015 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* XPS output device */
@@ -403,12 +403,14 @@ zip_new_info_node(gx_device_xps *xps_dev, const char *filename)
         gx_device_xps_f2i_t *f2i = xps_dev->f2i;
 #ifdef DEBUG
         int node = 1;
-#endif
         gx_device_xps_f2i_t *prev_f2i;
+#endif
         
         while (f2i != NULL) {
             if_debug2m('_', dev->memory, "node:%d %s\n", node++, f2i->filename);
+#ifdef DEBUG
             prev_f2i = f2i;
+#endif
             f2i=f2i->next;
         }
         if_debug1m('_', dev->memory, "tail okay=%s\n", prev_f2i == xps_dev->f2i_tail ? "yes" : "no");
@@ -1825,7 +1827,8 @@ xps_begin_image(gx_device *dev, const gs_gstate *pgs,
         csindex == gs_color_space_index_DeviceN))
         goto use_default;
 
-    gs_matrix_invert(&pim->ImageMatrix, &mat);
+    if (gs_matrix_invert(&pim->ImageMatrix, &mat) < 0)
+        goto use_default;
     if (pgs)
         gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
 
