@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -125,6 +125,7 @@ alloc_indexed_cspace(pcl_cs_indexed_t ** ppindexed,
     if (code < 0) {
         free_indexed_cspace(pmem, pindexed,
                             "allocate pcl indexed color space");
+        gs_free_object(pmem, bp, "allocate pcl indexed color space");
         return code;
     }
 
@@ -986,13 +987,17 @@ pcl_cs_indexed_build_cspace(pcl_state_t * pcs,
             }
         }
     }
-    pcl_cs_indexed_set_norm_and_Decode(ppindexed,
+    code = pcl_cs_indexed_set_norm_and_Decode(ppindexed,
                                        wht_ref[0], wht_ref[1], wht_ref[2],
                                        blk_ref[0], blk_ref[1], blk_ref[2]
         );
+    if (code < 0)
+        return code;
 
     /* set the palette size and the default palette entries */
-    pcl_cs_indexed_set_num_entries(ppindexed, 1L << bits, gl2);
+    code = pcl_cs_indexed_set_num_entries(ppindexed, 1L << bits, gl2);
+    if (code < 0)
+        return code;
 
     /* now can indicate if the palette is fixed */
     pindexed->pfixed = pfixed;
@@ -1060,10 +1065,13 @@ pcl_cs_indexed_build_special(pcl_cs_indexed_t ** ppindexed,
     pindexed->cid = cid;
     pindexed->num_entries = 2;
     /* set up the normalization information - not strictly necessary */
-    pcl_cs_indexed_set_norm_and_Decode(ppindexed,
+    code = pcl_cs_indexed_set_norm_and_Decode(ppindexed,
                                        wht_ref[0], wht_ref[1], wht_ref[2],
                                        blk_ref[0], blk_ref[1], blk_ref[2]
         );
+    if (code < 0)
+        return code;
+
     pindexed->Decode[1] = 1;
 
     for (i = 0; i < 3; i++) {
