@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -58,13 +58,13 @@ typedef enum
 
 /* Delete a soft font.  If it is the currently selected font, or the */
 /* current primary or secondary font, cause a new one to be chosen. */
-static void
+static int
 pcl_delete_soft_font(pcl_state_t * pcs, const byte * key, uint ksize,
                      void *value)
 {
     if (value == NULL) {
         if (!pl_dict_find_no_stack(&pcs->soft_fonts, key, ksize, &value))
-            return;             /* not a defined font */
+            return 0;             /* not a defined font */
     }
     {
         pl_font_t *plfontp = (pl_font_t *) value;
@@ -80,11 +80,16 @@ pcl_delete_soft_font(pcl_state_t * pcs, const byte * key, uint ksize,
                                                                plfontp->
                                                                params.
                                                                pjl_font_number)
-                > 0)
-                pcl_set_current_font_environment(pcs);
+                > 0) {
+                int code = pcl_set_current_font_environment(pcs);
+
+                if (code < 0)
+                    return code;
+            }
         pcs->font = pcs->font_selection[pcs->font_selected].font;
         pl_dict_undef_purge_synonyms(&pcs->soft_fonts, key, ksize);
     }
+    return 0;
 }
 
 /* ------ Commands ------ */
@@ -131,14 +136,14 @@ pcl_make_resident_font_copy(pcl_state_t * pcs)
         }
     if (found == false)
         return -1;
-    pl_dict_put_synonym(&pcs->built_in_fonts, key.data,
+    return pl_dict_put_synonym(&pcs->built_in_fonts, key.data,
                         key.size, current_font_id, current_font_id_size);
-    return 0;
 }
 
 static int                      /* ESC * c <fc_enum> F */
 pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
 {
+    int code = 0;
     gs_const_string key;
     void *value;
     pl_dict_enum_t denum;
@@ -149,7 +154,9 @@ pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
             pcs->font = pcs->font_selection[pcs->font_selected].font;
             pl_dict_enum_stack_begin(&pcs->soft_fonts, &denum, false);
             while (pl_dict_enum_next(&denum, &key, &value)) {
-                pcl_delete_soft_font(pcs, key.data, key.size, value);
+                code = pcl_delete_soft_font(pcs, key.data, key.size, value);
+                if (code < 0)
+                    return code;
                 /* when deleting fonts we must restart the enumeration
                    each time because the last deleted font may have had
                    deleted synonyms which are not properly registered in
@@ -163,12 +170,15 @@ pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
             /* Delete all temporary soft fonts. */
             pl_dict_enum_stack_begin(&pcs->soft_fonts, &denum, false);
             while (pl_dict_enum_next(&denum, &key, &value))
-                if (((pl_font_t *) value)->storage == pcds_temporary)
-                    pcl_delete_soft_font(pcs, key.data, key.size, value);
+                if (((pl_font_t *) value)->storage == pcds_temporary) {
+                    code = pcl_delete_soft_font(pcs, key.data, key.size, value);
+                    if (code < 0)
+                        return code;
+                }
             break;
         case 2:
             /* Delete soft font <font_id>. */
-            pcl_delete_soft_font(pcs, current_font_id, current_font_id_size,
+            code = pcl_delete_soft_font(pcs, current_font_id, current_font_id_size,
                                  NULL);
             /* decache the currently selected font in case we deleted it. */
             pcl_decache_font(pcs, -1, true);
@@ -204,8 +214,6 @@ pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
             break;
         case 6:
             {
-                int code;
-
                 if (pcs->font == 0) {
                     code = pcl_recompute_font(pcs, false);
                     if (code < 0)
@@ -230,11 +238,13 @@ pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
                     if (code < 0)
                         return code;
 
-                    pcl_delete_soft_font(pcs, current_font_id,
+                    code = pcl_delete_soft_font(pcs, current_font_id,
                                          current_font_id_size, NULL);
+                    if (code < 0)
+                        return code;
                     plfont->storage = pcds_temporary;
                     plfont->data_are_permanent = false;
-                    pl_dict_put(&pcs->soft_fonts, current_font_id,
+                    code = pl_dict_put(&pcs->soft_fonts, current_font_id,
                                 current_font_id_size, plfont);
                 }
             }
@@ -242,7 +252,7 @@ pcl_font_control(pcl_args_t * pargs, pcl_state_t * pcs)
         default:
             return 0;
     }
-    return 0;
+    return code;
 }
 
 static int                      /* ESC ) s <count> W */
@@ -320,12 +330,17 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
         }
     }
     /* Delete any previous font with this ID. */
-    pcl_delete_soft_font(pcs, current_font_id, current_font_id_size, NULL);
+    code = pcl_delete_soft_font(pcs, current_font_id, current_font_id_size, NULL);
+    if (code < 0)
+        return code;
     /* Create the generic font information. */
     plfont = pl_alloc_font(mem, "pcl_font_header(pl_font_t)");
     header = gs_alloc_bytes(mem, count, "pcl_font_header(header)");
-    if (plfont == 0 || header == 0)
+    if (plfont == NULL || header == NULL) {
+        gs_free_object(mem, header, "pcl_font_header(header)");
+        gs_free_object(mem, plfont, "pcl_font_header(pl_font_t)");
         return_error(e_Memory);
+    }
     memcpy(header, data, count);
     plfont->storage = pcds_temporary;
     plfont->data_are_permanent = false;
@@ -344,7 +359,7 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
     code = pl_font_alloc_glyph_table(plfont, 256, mem,
                                      "pcl_font_header(char table)");
     if (code < 0)
-        return code;
+        goto fail;
     /* Create the actual font. */
     switch (fst) {
         case plfst_bitmap:
@@ -354,13 +369,15 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
               bitmap:pfont = gs_alloc_struct(mem, gs_font_base, &st_gs_font_base,
                                         "pcl_font_header(bitmap font)");
 
-                if (pfont == 0)
-                    return_error(e_Memory);
+                if (pfont == NULL) {
+                    code = e_Memory;
+                    goto fail;
+                }
                 code =
                     pl_fill_in_font((gs_font *) pfont, plfont, pcs->font_dir,
                                     mem, "nameless_font");
                 if (code < 0)
-                    return code;
+                    goto fail;
                 pl_fill_in_bitmap_font(pfont, gs_next_ids(mem, 1));
                 /* Extract parameters from the font header. */
                 if (pfh->HeaderFormat == pcfh_resolution_bitmap) {
@@ -411,7 +428,7 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
                                               pfh->HeaderFormat ==
                                               pcfh_truetype_large, &errors);
                     if (code < 0)
-                        return code;
+                        goto fail;
                     /* truetype large format 16 can be truetype or bitmap -
                        absurd */
                     if ((pfh->HeaderFormat == pcfh_truetype_large) &&
@@ -422,8 +439,10 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
                 pfont =
                     gs_alloc_struct(mem, gs_font_type42, &st_gs_font_type42,
                                     "pcl_font_header(truetype font)");
-                if (pfont == 0)
-                    return_error(e_Memory);
+                if (pfont == NULL) {
+                    code = e_Memory;
+                    goto fail;
+                }
 
                 {
                     uint num_chars = pl_get_uint16(pfh->LastCode);
@@ -435,14 +454,16 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
                     code = pl_tt_alloc_char_glyphs(plfont, num_chars, mem,
                                                    "pcl_font_header(char_glyphs)");
                     if (code < 0)
-                        return code;
+                        goto fail;
                 }
                 code =
                     pl_fill_in_font((gs_font *) pfont, plfont, pcs->font_dir,
                                     mem, "nameless_font");
                 if (code < 0)
-                    return code;
-                pl_fill_in_tt_font(pfont, NULL, gs_next_ids(mem, 1));
+                    goto fail;
+                code = pl_fill_in_tt_font(pfont, NULL, gs_next_ids(mem, 1));
+                if (code < 0)
+                    goto fail;
                 {
                     uint pitch_cp =
                         (uint) (pl_get_uint16(pfh->Pitch) * 1000.0 /
@@ -457,13 +478,15 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
                     gs_alloc_struct(mem, gs_font_base, &st_gs_font_base,
                                     "pcl_font_header(bitmap font)");
 
-                if (pfont == 0)
-                    return_error(e_Memory);
+                if (pfont == NULL) {
+                    code = e_Memory;
+                    goto fail;
+                }
                 code =
                     pl_fill_in_font((gs_font *) pfont, plfont, pcs->font_dir,
                                     mem, "nameless_font");
                 if (code < 0)
-                    return code;
+                    goto fail;
                 pl_fill_in_intelli_font(pfont, gs_next_ids(mem, 1));
                 {
                     uint pitch_cp =
@@ -473,29 +496,40 @@ pcl_font_header(pcl_args_t * pargs, pcl_state_t * pcs)
                 break;
             }
         default:
-            return_error(gs_error_invalidfont); /* can't happen */
+            code = gs_error_invalidfont; /* can't happen */
+            goto fail;
     }
     /* Extract common parameters from the font header. */
     plfont->params.symbol_set = pl_get_uint16(pfh->SymbolSet);
-    if (pfh->Spacing > 1)
-        return_error(e_Range);
+    if (pfh->Spacing > 1) {
+        code = e_Range;
+        goto fail;
+    }
     plfont->params.proportional_spacing = pfh->Spacing;
     plfont->params.style = (pfh->StyleMSB << 8) + pfh->StyleLSB;
     plfont->params.stroke_weight =      /* signed byte */
         (int)(pfh->StrokeWeight ^ 0x80) - 0x80;
     plfont->params.typeface_family =
         (pfh->TypefaceMSB << 8) + pfh->TypefaceLSB;
-    pl_dict_put(&pcs->soft_fonts, current_font_id,
+    plfont->params.pjl_font_number = pcs->pjl_dlfont_number++;
+    
+    code = pl_dict_put(&pcs->soft_fonts, current_font_id,
                 current_font_id_size, plfont);
+    if (code < 0)
+        goto fail;
     plfont->pfont->procs.define_font = gs_no_define_font;
 
     if ((code = gs_definefont(pcs->font_dir, plfont->pfont)) != 0) {
-        return (code);
+        goto fail;
     }
 
     if (plfont->scaling_technology == plfst_TrueType)
         code = pl_fapi_passfont(plfont, 0, NULL, NULL, NULL, 0);
 
+fail:
+    if (code < 0)
+        pl_free_font(mem, plfont, "pcl_font_header(pl_font_t)");
+
     return (code);
 }
 
@@ -977,7 +1011,7 @@ pcl_alphanumeric_id_data(pcl_args_t * pargs, pcl_state_t * pcs)
         case 20:
             /* deletes the font association named by the current Font ID */
             if (pcs->font_id_type == string_id)
-                pcl_delete_soft_font(pcs, current_font_id,
+                return pcl_delete_soft_font(pcs, current_font_id,
                                      current_font_id_size, NULL);
             break;
         case 21:
@@ -1030,7 +1064,7 @@ pcsfont_do_registration(pcl_parser_state_t * pcl_parser_state,
                                   pcl_alphanumeric_id_data, pca_bytes)
         return 0;
 }
-static void
+static int
 pcsfont_do_reset(pcl_state_t * pcs, pcl_reset_type_t type)
 {
     if (type & (pcl_reset_initial | pcl_reset_printer | pcl_reset_overlay)) {
@@ -1040,24 +1074,28 @@ pcsfont_do_reset(pcl_state_t * pcs, pcl_reset_type_t type)
         pcs->character_code = 0;
         pcs->font_id_type = numeric_id;
         if ((type & pcl_reset_printer) != 0) {
+            int code = 0;
             pcl_args_t args;
 
             arg_set_uint(&args, 1);     /* delete temporary fonts */
-            pcl_font_control(&args, pcs);
+            code = pcl_font_control(&args, pcs);
             if (pcs->alpha_font_id.id != 0)
                 gs_free_object(pcs->memory,
                                pcs->alpha_font_id.id, "pcsfont_do_reset");
+            if (code < 0)
+                return code;
         }
         pcs->alpha_font_id.id = 0;
     }
+    return 0;
 }
 static int
 pcsfont_do_copy(pcl_state_t * psaved, const pcl_state_t * pcs,
                 pcl_copy_operation_t operation)
 {
     if (operation & pcl_copy_after) {   /* Don't restore the soft font set. */
-            /**** MUST HANDLE POSSIBILITY THAT CURRENT FONT WAS DELETED. ****/
         psaved->soft_fonts = pcs->soft_fonts;
+        psaved->built_in_fonts = pcs->built_in_fonts;
     }
     return 0;
 }
