@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -211,6 +211,7 @@ get_next_char(pcl_state_t * pcs,
     int len = *plen;
     pl_font_t *plfont = pcs->font;
     bool substituting = false;
+    int code = 0;
     gs_char chr;
     gs_char mapped_chr;         /* NB wrong type */
     bool db;
@@ -291,7 +292,9 @@ get_next_char(pcl_state_t * pcs,
      */
     if (!substituting && substituting_allowed(pcs, db ? mapped_chr : chr)) {
         pcl_decache_font(pcs, -1, true);
-        pcl_recompute_font(pcs, true);
+        code = pcl_recompute_font(pcs, true);
+        if (code < 0)
+            return code;
         substituting = true;
         *unstyled_substitution = true;
         plfont = pcs->font;
@@ -303,7 +306,9 @@ get_next_char(pcl_state_t * pcs,
        Restore the old font */
     if (substituting) {
         pcl_decache_font(pcs, -1, true);
-        pcl_recompute_font(pcs, false);
+        code = pcl_recompute_font(pcs, false);
+        if (code < 0)
+            return code;
         set_gs_font(pcs);
     }
 
@@ -498,11 +503,10 @@ show_char_foreground(const pcl_state_t * pcs, const gs_char * pbuff)
     text.data.chars = pbuff;
     text.size = 1;
     code = gs_text_begin(pcs->pgs, &text, pcs->memory, &penum);
-    if (code >= 0)
+    if (code >= 0) {
         code = gs_text_process(penum);
-
-    if (code >= 0)
         gs_text_release(penum, "show_char_foreground");
+    }
     return code;
 }
 
@@ -609,11 +613,18 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
     int code = 0;
 
     /* save the graphic state and set the background raster operation */
-    pcl_gsave(pcs);
-    if (pcs->pattern_transparent)
-        pcl_set_drawing_color(pcs, pcl_pattern_solid_white, 0, false);
-    gs_setrasterop(pgs, (gs_rop3_t) rop3_know_S_1((int)rop));
-    gs_currentpoint(pgs, &pt);
+    code = pcl_gsave(pcs);
+    if (code < 0)
+        return code;
+    if (pcs->pattern_transparent) {
+        code = pcl_set_drawing_color(pcs, pcl_pattern_solid_white, 0, false);
+        if (code < 0)
+            return code;
+    }
+    if (((code = gs_setrasterop(pgs, (gs_rop3_t) rop3_know_S_1((int)rop))) < 0) ||
+        ((code = gs_currentpoint(pgs, &pt)) < 0)) {
+        return code;
+    }
 
     if (plfont->scaling_technology == plfst_bitmap) {
         gs_char chr = pbuff[0];
@@ -626,8 +637,7 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
 
         /* empty characters have no background */
         if (cdata == 0) {
-            pcl_grestore(pcs);
-            return 0;
+            return pcl_grestore(pcs);
         }
 
         /* allocate the image enumerator */
@@ -663,8 +673,10 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
         gs_text_enum_t *penum;
 
         /* clear the path; start the new one from the current point */
-        gs_newpath(pgs);
-        gs_moveto(pgs, pt.x, pt.y);
+        if (((code = gs_newpath(pgs)) < 0) ||
+            ((code = gs_moveto(pgs, pt.x, pt.y)) < 0)) {
+            return code;
+        }
         text.data.chars = pbuff;
         text.size = 1;
         text.operation =
@@ -674,31 +686,41 @@ show_char_background(pcl_state_t * pcs, const gs_char * pbuff)
             code = gs_text_process(penum);
         if (code >= 0) {
             /* append the characters bounding box and use eofill */
-            gs_pathbbox(pgs, &bbox);
-            gs_rectappend(pgs, &bbox, 1);
-            gs_eofill(pgs);
-            gs_text_release(penum, "show_char_background");
+            if ((code = gs_pathbbox(pgs, &bbox)) >= 0 &&
+                (code = gs_rectappend(pgs, &bbox, 1)) >= 0 &&
+                (code = gs_eofill(pgs)) >= 0)
+            {
+                gs_text_release(penum, "show_char_background");
+            }
+            else
+                return code;
         }
     }
 
-    pcl_grestore(pcs);
-    return code;
+    return pcl_grestore(pcs);
 }
 
 /*
  * get the advance width.
  */
-static double
+static int
 pcl_get_width(pcl_state_t * pcs, gs_point * advance_vector,
               const gs_point * pscale, gs_char chr, bool is_space,
-              bool print_undefined)
+              bool print_undefined, double *output_width)
 {
+    int code = 0;
     pcl_font_selection_t *pfp = &(pcs->font_selection[pcs->font_selected]);
     double width;
 
     if (chr != 0xffff || print_undefined) {
         if (!pfp->params.proportional_spacing || is_space)
-            width = pcl_hmi(pcs);
+        {
+            code = pcl_update_hmi_cp(pcs);
+            if (code < 0)
+                return code;
+
+            width = pcs->hmi_cp;
+        }
         else {
             if (pcs->font->scaling_technology == plfst_TrueType ||
                 pcs->font->scaling_technology == plfst_MicroType) {
@@ -714,12 +736,17 @@ pcl_get_width(pcl_state_t * pcs, gs_point * advance_vector,
             width += (double) pcs->uom_cp / 2.0;
             width -= fmod(width, (double) pcs->uom_cp);
         }
-    } else if (is_space)
-        width = pcl_hmi(pcs);
+    } else if (is_space) {
+        code = pcl_update_hmi_cp(pcs);
+        if (code < 0)
+            return code;
+        width = pcs->hmi_cp;
+    }
     else
         width = 0.0;
     /* round to nearest integral pcl units */
-    return width;
+    *output_width = width;
+    return code;
 }
 
 /*
@@ -812,9 +839,11 @@ pcl_show_chars_slow(pcl_state_t * pcs,
         buff[0] = chr;
 
         /* round width to integral pcl current units */
-        width =
+        code =
             (pcl_get_width
-             (pcs, &advance_vector, pscale, chr, is_space, print_undefined));
+             (pcs, &advance_vector, pscale, chr, is_space, print_undefined, &width));
+        if (code < 0)
+            return code;
 
         /*
          * Check for transitions of the left margin; this check is
@@ -881,7 +910,9 @@ pcl_show_chars_slow(pcl_state_t * pcs,
             else
                 tmp_x += (pcs->last_width - width) / 2;
         }
-        gs_moveto(pgs, tmp_x / pscale->x, cpt.y / pscale->y);
+        code = gs_moveto(pgs, tmp_x / pscale->x, cpt.y / pscale->y);
+        if (code < 0)
+            return code;
 
         if (chr != 0xffff || print_undefined) {
             /* if source is opaque, show and opaque background */
@@ -901,7 +932,8 @@ pcl_show_chars_slow(pcl_state_t * pcs,
             if (code < 0)
                 break;
 
-            pcl_mark_page_for_character(pcs, &pt);
+            if ((code = pcl_mark_page_for_character(pcs, &pt)) < 0)
+                return code;
         }
 
         /*
@@ -926,7 +958,9 @@ pcl_show_chars_slow(pcl_state_t * pcs,
         }
         if (unstyled_substitution) {
             pcl_decache_font(pcs, -1, true);
-            pcl_recompute_font(pcs, false);
+            code = pcl_recompute_font(pcs, false);
+            if (code < 0)
+                return code;
             set_gs_font(pcs);
         }
     }
@@ -1023,9 +1057,7 @@ pcl_text(const byte * str, uint size, pcl_state_t * pcs, bool literal)
     set_gs_font(pcs);
 
     gs_currentmatrix(pgs, &user_ctm);
-
-    (void)pcl_font_scale(pcs, &scale);
-
+    pcl_font_scale(pcs, &scale);
     gs_scale(pgs, scale.x, scale.y);
 
     /*
@@ -1079,7 +1111,9 @@ pcl_do_underline(pcl_state_t * pcs)
         int code;
 
         /* save the grapics state */
-        pcl_gsave(pcs);
+        code = pcl_gsave(pcs);
+        if (code < 0)
+            return;
 
         code = pcl_set_drawing_color(pcs,
                                      pcs->pattern_type,
@@ -1095,9 +1129,11 @@ pcl_do_underline(pcl_state_t * pcs)
          * at 300 dpi only)
          */
         gs_setlinewidth(pgs, dots(3));
-        gs_moveto(pgs, pcs->underline_start.x, y);
-        gs_lineto(pgs, pcs->cap.x, y);
-        gs_stroke(pgs);
+        if ((gs_moveto(pgs, pcs->underline_start.x, y) < 0) ||
+            (gs_lineto(pgs, pcs->cap.x, y) < 0) ||
+            (gs_stroke(pgs) < 0)) {
+            return;
+        }
 
         pcl_grestore(pcs);
     }
@@ -1284,7 +1320,7 @@ pctext_do_registration(pcl_parser_state_t * pcl_parser_state,
     return 0;
 }
 
-static void
+static int
 pctext_do_reset(pcl_state_t * pcs, pcl_reset_type_t type)
 {
     static const uint mask = (pcl_reset_initial
@@ -1297,6 +1333,7 @@ pctext_do_reset(pcl_state_t * pcs, pcl_reset_type_t type)
         pcs->text_parsing_method = tpm_0_SBCS;
         pcs->text_path = 0;
     }
+    return 0;
 }
 
 const pcl_init_t pctext_init = { pctext_do_registration, pctext_do_reset, 0 };
