@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -975,7 +975,7 @@ process_ccitt_compress(gs_gstate * pgs,
         state.K = -1;
     s_CFD_template.init((stream_state *) & state);
     scr.ptr = pin + 4 - 1;
-    scr.limit = scr.ptr + insize;
+    scr.limit = scr.ptr + insize - 4;
 
     wrsize = (state.Columns + 7) / 8;
     temp_buffer = gs_alloc_bytes(prast->pmem, wrsize, "CCITT temp_buffer");
