@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -7,8 +7,8 @@
    This software is distributed under license and may not be copied, modified
    or distributed except as expressly authorized under the terms of that
    license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   or contact Artifex Software, Inc.,  1305 Grant Avenue - Suite 200,
+   Novato, CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Font API support  */
@@ -651,15 +651,8 @@ pl_fapi_passfont(pl_font_t * plfont, int subfont, char *fapi_request,
                          (gs_fapi_get_server_param_callback)
                          pl_get_server_param);
 
-    if (pfont->FontType == ft_MicroType && code < 0) {
-        return (code);
-    }
-
-    /* For now, we'll fall back to AFS
-       We should return an error in the future.
-     */
     if (code < 0 || fapi_id == NULL) {
-        return (0);
+        return code;
     }
 
     pfont->procs.build_char = pl_fapi_build_char;
