@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -461,15 +461,20 @@ pl_glyph_name(gs_font * pfont, gs_glyph glyph, gs_const_string * pstr)
     return 0;
 }
 
+#include "assert_.h"
+
 /* Get the unicode valude for a glyph */
 static int
 pl_decode_glyph(gs_font * font, gs_glyph glyph, int ch, ushort *unicode_return, unsigned int length)
 {
-    pl_font_t *plfont = (pl_font_t *) font->client_data;
+
+    if (ch < 0 || ch > 255)
+        return (int) GS_NO_CHAR;
 
     if (length == 0)
         return 1;
-    *unicode_return = plfont->last_char;
+
+    *unicode_return = (ushort)ch;
     return 1;
 }
 
@@ -488,7 +493,7 @@ pl_font_glyph_width_cache_node_add(pl_font_t *plfont,
     if (plfont->widths_cache_nitems > PL_MAX_WIDTHS_CACHE_NITEMS) {
         pl_font_glyph_width_cache_remove_nodes(plfont);
     }
-    
+
     node = (pl_glyph_width_node_t *) gs_alloc_bytes(plfont->pfont->memory,
                                                  sizeof
                                                  (pl_glyph_width_node_t),
@@ -559,7 +564,7 @@ pl_font_char_width(const pl_font_t * plfont, const void *pgs,
     if (pl_font_glyph_width_cache_node_search(plfont, char_code, pwidth) >= 0) {
         return(code);
     }
-    
+
     if ((code = (*(plfont)->char_width) (plfont, pgs, char_code, pwidth)) == 0) {
 
         /* at least here, ignore the return value - if we fail to add a node
@@ -1167,7 +1172,7 @@ pl_load_resident_font_data_from_file(gs_memory_t * mem, pl_font_t * plfont)
     byte *data;
 
     if (plfont->font_file && !plfont->font_file_loaded) {
-        stream *in = sfopen(plfont->font_file, gp_fmode_rb, mem);
+        stream *in = sfopen(plfont->font_file, "r", mem);
 
         if (in == NULL)
             return -1;
