@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -306,7 +306,7 @@ pl_load_ufst_lineprinter(gs_memory_t * mem, pl_dict_t * pfontdict,
                    resident_table[i].character_complement, 8);
 
             if (use_unicode_names_for_keys)
-                pl_dict_put(pfontdict,
+                code = pl_dict_put(pfontdict,
                             (const byte *)resident_table[i].unicode_fontname,
                             32, pplfont);
             else {
@@ -314,8 +314,10 @@ pl_load_ufst_lineprinter(gs_memory_t * mem, pl_dict_t * pfontdict,
 
                 key[2] = (byte) i;
                 key[0] = key[1] = 0;
-                pl_dict_put(pfontdict, key, sizeof(key), pplfont);
+                code = pl_dict_put(pfontdict, key, sizeof(key), pplfont);
             }
+            if (code < 0)
+                return code;
             pplfont->storage = storage; /* should be an internal font */
             pplfont->data_are_permanent = true;
             pplfont->header = (byte *) header;
@@ -462,6 +464,7 @@ pl_load_built_in_mtype_fonts(const char *pathname, gs_memory_t * mem,
                 uint spaceBand;
                 uint scaleFactor;
                 bool used = false;
+                int code = 0;
 
                 /* For Microtype fonts, once we get here, these
                  * pl_fapi_get*() calls cannot fail, so we can
@@ -491,15 +494,11 @@ pl_load_built_in_mtype_fonts(const char *pathname, gs_memory_t * mem,
                 }
 
                 for (j = 0; strlen(resident_table[j].full_font_name); j++) {
-                    uint pitch_cp;
-
                     if (strcmp
                         ((char *)resident_table[j].full_font_name,
                          (char *)pname) != 0)
                         continue;
 
-                    pitch_cp = (uint)((spaceBand * 100.0) / scaleFactor + 0.5);
-
 #ifdef DEBUG
                     if (gs_debug_c('='))
                         dmprintf2(mem, "Loading %s from fco %s\n", pname,
@@ -510,16 +509,7 @@ pl_load_built_in_mtype_fonts(const char *pathname, gs_memory_t * mem,
 
                     if (scaleFactor == 8782) {
                         plfont->pts_per_inch = 72.307f;
-                        pitch_cp = (uint)((spaceBand * 100 * 72.0) /
-                                          (scaleFactor * 72.307) + 0.5);
                     }
-#ifdef DEBUG
-                    if (gs_debug_c('='))
-                        dmprintf3(mem,
-                                  "scale factor=%d, pitch (cp)=%d per_inch_x100=%d\n",
-                                  scaleFactor, pitch_cp,
-                                  (uint) (720000.0 / pitch_cp));
-#endif
 
                     plfont->font_type = resident_table[j].font_type;
                     plfont->storage = storage;
@@ -549,14 +539,16 @@ pl_load_built_in_mtype_fonts(const char *pathname, gs_memory_t * mem,
                         continue;
                     }
                     if (use_unicode_names_for_keys)
-                        pl_dict_put(pfontdict,
+                        code = pl_dict_put(pfontdict,
                                     (const byte *)resident_table[j].
                                     unicode_fontname, 32, plfont);
                     else {
                         key[2] = (byte) j;
                         key[0] = key[1] = 0;
-                        pl_dict_put(pfontdict, key, sizeof(key), plfont);
+                        code = pl_dict_put(pfontdict, key, sizeof(key), plfont);
                     }
+                    if (code < 0)
+                        return code;
                     used = true;
                 }
                 /* If we've stored the font, null the local reference */
@@ -692,6 +684,7 @@ pl_load_built_in_fonts(const char *pathname, gs_memory_t * mem,
             if (code < 0) {
                 dmprintf1(mem, "input output failure on TrueType File %s\n",
                           tmp_path_copy);
+                sfclose(in);
                 continue;
             }
 
@@ -699,6 +692,7 @@ pl_load_built_in_fonts(const char *pathname, gs_memory_t * mem,
                 dmprintf1(mem,
                           "could not extract font file name from file %s\n",
                           tmp_path_copy);
+                sfclose(in);
                 continue;
             }
 
@@ -713,16 +707,22 @@ pl_load_built_in_fonts(const char *pathname, gs_memory_t * mem,
                                     gs_next_ids(mem, 1), &plfont,
                                     buffer) < 0) {
                     /* vm error */
+                    gs_enumerate_files_close(fe);
                     return gs_throw1(0,
                                      "An unrecoverable failure occurred while reading the resident font %s\n",
                                      tmp_path_copy);
                 }
                 /* reopen the file */
                 in = sfopen(tmp_path_copy, "r", mem);
-                if (in == NULL)
+                if (in == NULL) {
+                    gs_free_object(mem, plfont->pfont, "pl_tt_load_font(gs_font_type42)");
+                    pl_free_tt_fontfile_buffer(mem, plfont->header);
+                    gs_free_object(mem, plfont, "pl_tt_load_font(pl_font_t)");
+                    gs_enumerate_files_close(fe);
                     return gs_throw1(0,
                                      "An unrecoverable failure occurred while reading the resident font %s\n",
                                      tmp_path_copy);
+                }
 
                 plfont->storage = storage;
                 plfont->data_are_permanent = false;
@@ -735,21 +735,27 @@ pl_load_built_in_fonts(const char *pathname, gs_memory_t * mem,
                 memcpy(plfont->character_complement,
                        residentp->character_complement, 8);
                 if (use_unicode_names_for_keys)
-                    pl_dict_put(pfontdict,
+                    code = pl_dict_put(pfontdict,
                                 (const byte *)residentp->unicode_fontname, 32,
                                 plfont);
                 else {
                     key[2] = (byte) (residentp - resident_table);
                     key[0] = key[1] = 0;
-                    pl_dict_put(pfontdict, key, sizeof(key), plfont);
+                    code = pl_dict_put(pfontdict, key, sizeof(key), plfont);
                     /* leave data stored in the file.  NB this should be a fatal error also. */
-                    if (pl_store_resident_font_data_in_file
+                    if ((code >= 0) && pl_store_resident_font_data_in_file
                         (tmp_path_copy, mem, plfont) < 0) {
                         dmprintf1(mem, "%s could not store data",
                                   tmp_path_copy);
                         continue;
                     }
                 }
+                if (code < 0) {
+                    gs_free_object(mem, plfont->pfont, "pl_tt_load_font(gs_font_type42)");
+                    pl_free_tt_fontfile_buffer(mem, plfont->header);
+                    gs_free_object(mem, plfont, "pl_tt_load_font(pl_font_t)");
+                    continue;
+                }
                 found = true;
                 found_any = true;
             }
@@ -767,6 +773,8 @@ pl_load_built_in_fonts(const char *pathname, gs_memory_t * mem,
                         get_name_from_tt_file(in, mem, buffer, WINDOWSNAME);
                     sfclose(in);
                     dmprintf1(mem, "Windows name %s\n", buffer);
+                    if (code < 0)
+                        return code;
                 }
 #endif
             }
