@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -343,7 +343,7 @@ static int pjl_dist_process_dict(gs_memory_t *mem, gs_c_param_list *plist, gs_pa
     if (*p1 == 0x00)
         return -1;
 
-    /* NULL out the dictioanry end marks */
+    /* NULL out the dictionary end marks */
     *p1++ = 0x00;
     if (*p1 == 0x00)
         return -1;
@@ -415,9 +415,7 @@ static int pjl_dist_process_dict_or_hexstring(gs_memory_t *mem, gs_c_param_list
     ps.data = (const byte *)p1;
     ps.size = strlen(p1);
     ps.persistent = false;
-    param_write_string((gs_param_list *)plist, key, &ps);
-
-    return 0;
+    return param_write_string((gs_param_list *)plist, key, &ps);
 }
 
 static int pjl_dist_process_name(gs_memory_t *mem, gs_c_param_list *plist, gs_param_name *key, char **p)
@@ -462,9 +460,7 @@ static int pjl_dist_process_string(gs_memory_t *mem, gs_c_param_list *plist, gs_
     ps.data = (const byte *)start;
     ps.size = strlen(start);
     ps.persistent = false;
-    param_write_string((gs_param_list *)plist, key, &ps);
-
-    return 0;
+    return param_write_string((gs_param_list *)plist, key, &ps);
 }
 
 /* We need to know how many items are in an array or dictionary */
@@ -903,6 +899,7 @@ static int pjl_dist_process_number(gs_memory_t *mem, gs_c_param_list *plist, gs_
 {
     char *start = *p;
     bool integer = true;
+    int code = 0;
 
     while (**p != ' ' && **p != 0x00) {
         if (**p == '.')
@@ -918,13 +915,13 @@ static int pjl_dist_process_number(gs_memory_t *mem, gs_c_param_list *plist, gs_
 
     if (!integer) {
         float f = (float)atof(start);
-        param_write_float((gs_param_list *)plist, key, (float *)&f);
+        code = param_write_float((gs_param_list *)plist, key, (float *)&f);
     } else {
         long i = atol(start);
-        param_write_long((gs_param_list *)plist, key, &i);
+        code = param_write_long((gs_param_list *)plist, key, &i);
     }
 
-    return 0;
+    return code;
 }
 
 /* Given a string to parse, parse it and add what we find to the supplied
@@ -944,7 +941,7 @@ static int pjl_dist_add_tokens_to_list(gs_param_list *plist, char **p)
             case 'f':
                 if (strncmp(p1, "false", 5) == 0) {
                     bool t = false;
-                    param_write_bool((gs_param_list *)plist, key, &t);
+                    code = param_write_bool((gs_param_list *)plist, key, &t);
                     p1 += 5;
                     key = NULL;
                 } else {
@@ -954,7 +951,7 @@ static int pjl_dist_add_tokens_to_list(gs_param_list *plist, char **p)
             case 't':
                 if (strncmp(p1, "true", 4) == 0) {
                     bool t = true;
-                    param_write_bool((gs_param_list *)plist, key, &t);
+                    code = param_write_bool((gs_param_list *)plist, key, &t);
                     p1 += 4;
                     key = NULL;
                 } else {
@@ -1021,6 +1018,8 @@ int pcl_pjl_setdistillerparams(gs_memory_t *mem, gx_device *device, char *distil
     gs_c_param_list *plist;
 
     plist = gs_c_param_list_alloc(mem, "temp C param list for PJL distillerparams");
+    if (plist == NULL)
+        return -1;
     gs_c_param_list_write(plist, mem);
     gs_param_list_set_persistent_keys((gs_param_list *) plist, false);
     gs_c_param_list_write_more(plist);
