@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -24,32 +24,32 @@ int
 main(int argc, char *argv[])
 {
     int code, code1;
-    void *minst;
+    void *minst = NULL;
+    size_t uel_len = strlen(PJL_UEL);
     
-    code = plapi_new_instance(&minst, (void *)0);
+    code = gsapi_new_instance(&minst, (void *)0);
     if (code < 0)
-	return EXIT_FAILURE;
+        return EXIT_FAILURE;
 
-    if (code == 0)
-        code = plapi_init_with_args(minst, argc, argv);
+    if (code >= 0)
+        code = gsapi_init_with_args(minst, argc, argv);
 
-    if (code == 0) {
-        size_t uel_len = strlen(PJL_UEL);
-        if (
-            (code = plapi_run_string_begin(minst) < 0) ||
-            (code = plapi_run_string_continue(minst, PJL_UEL, uel_len) < 0) ||
-            (code = plapi_run_string_end(minst) < 0)
-            )
-            ; /* nothing */
-    }
+    if (code >= 0)
+        code = gsapi_run_string_begin(minst);
+    if (code >= 0)
+        code = gsapi_run_string_continue(minst, PJL_UEL, uel_len);
+    if (code >= 0)
+        code = gsapi_run_string_end(minst);
+    if (code == gs_error_InterpreterExit)
+        code = 0;
 
-    code1 = plapi_exit(minst);
+    code1 = gsapi_exit(minst);
     if ((code == 0) || (code == gs_error_Quit))
-	code = code1;
+        code = code1;
 
-    plapi_delete_instance(minst);
+    gsapi_delete_instance(minst);
 
     if ((code == 0) || (code == gs_error_Quit))
-	return EXIT_SUCCESS;
+        return EXIT_SUCCESS;
     return EXIT_FAILURE;
 }
