@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -254,9 +254,11 @@ int
 px_initgraphics(px_state_t * pxs)
 {
     gs_gstate *pgs = pxs->pgs;
+    int code;
 
     px_gstate_reset(pxs->pxgs);
-    gs_initgraphics(pgs);
+    code = gs_initgraphics(pgs);
+    if (code < 0) return code;
 
     gs_setfilladjust(pgs, 0.5, 0.5);
 
@@ -273,7 +275,8 @@ px_initgraphics(px_state_t * pxs)
 
         /* We need the H-P interpretation of zero-length lines */
         /* and of using bevel joins for the segments of flattened curves. */
-        (void)gs_setdotlength(pgs, 72.0 / 300, true);
+        code = gs_setdotlength(pgs, 72.0 / 300, true);
+        if (code < 0) return code;
     }
     /* we always clamp coordinates hp does not seem to report
        limit checks in paths */
