@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,6 +18,10 @@
 
 #ifndef iconf_INCLUDED
 #define iconf_INCLUDED
+
+#include "stdpre.h"
+#include "gxiodev.h"
+
 /* ------ Imported data ------ */
 
 /* Configuration information imported from iccinit[01].c */
