@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,10 +19,10 @@
 #ifndef iddstack_INCLUDED
 #  define iddstack_INCLUDED
 
-#ifndef dict_stack_DEFINED
-#  define dict_stack_DEFINED
+#include "stdpre.h"
+#include "iref.h"
+
 typedef struct dict_stack_s dict_stack_t;
-#endif
 
 /*
  * Reset the cached top values.  Every routine that alters the
