@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,15 +19,10 @@
 #ifndef idparam_INCLUDED
 #  define idparam_INCLUDED
 
-#ifndef gs_matrix_DEFINED
-#  define gs_matrix_DEFINED
-typedef struct gs_matrix_s gs_matrix;
-#endif
-
-#ifndef gs_uid_DEFINED
-#  define gs_uid_DEFINED
-typedef struct gs_uid_s gs_uid;
-#endif
+#include "stdpre.h"
+#include "iref.h"
+#include "gsmatrix.h"
+#include "gsuid.h"
 
 /*
  * Unless otherwise noted, all the following routines return 0 for
