@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -38,7 +38,7 @@
 #include "oper.h"
 #include "iconf.h"              /* for gs_init_* imports */
 #include "idebug.h"
-#include "idict.h"
+#include "iddict.h"
 #include "iname.h"              /* for name_init */
 #include "dstack.h"
 #include "estack.h"
@@ -59,9 +59,7 @@
 #include "iplugin.h"
 #include "zfile.h"
 
-#ifdef PACIFY_VALGRIND
 #include "valgrind.h"
-#endif
 
 /* ------ Exported data ------ */
 
@@ -87,6 +85,7 @@ gs_main_alloc_instance(gs_memory_t *mem)
                                                          "init_main_instance");
     if (minst == NULL)
         return NULL;
+    memset(minst, 0, sizeof(gs_main_instance));
     memcpy(minst, &gs_main_instance_init_values, sizeof(gs_main_instance_init_values));
     minst->heap = mem;
     mem->gs_lib_ctx->top_of_system = minst;
@@ -128,6 +127,10 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
               int max_lib_paths)
 {
     ref *array;
+    int code = 0;
+
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf1(minst->heap, "%% Init phase 0 started, instance 0x%p\n", minst);
 
     /* Do platform-dependent initialization. */
     /* We have to do this as the very first thing, */
@@ -151,18 +154,25 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
                                         "lib_path array");
     if (array == 0) {
         gs_lib_finit(1, gs_error_VMerror, minst->heap);
-        return_error(gs_error_VMerror);
+        code = gs_note_error(gs_error_VMerror);
+        goto fail;
     }
     make_array(&minst->lib_path.container, avm_foreign, max_lib_paths,
                array);
     make_array(&minst->lib_path.list, avm_foreign | a_readonly, 0,
                minst->lib_path.container.value.refs);
-    minst->lib_path.env = 0;
-    minst->lib_path.final = 0;
+    minst->lib_path.env = NULL;
+    minst->lib_path.final = NULL;
     minst->lib_path.count = 0;
+    minst->lib_path.first_is_current = 0;
     minst->user_errors = 1;
     minst->init_done = 0;
-    return 0;
+
+fail:
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf2(minst->heap, "%% Init phase 0 %s, instance 0x%p\n", code < 0 ? "failed" : "done", minst);
+
+    return code;
 }
 
 /* Initialization to be done before constructing any objects. */
@@ -171,51 +181,59 @@ gs_main_init1(gs_main_instance * minst)
 {
     gs_dual_memory_t idmem;
     name_table *nt = NULL;
-    int code = 0;
+    int code;
 
-    if (minst->init_done < 1) {
-        code = ialloc_init(&idmem, minst->heap,
-                        minst->memory_clump_size, gs_have_level2());
+    if (minst->init_done >= 1)
+        return 0;
 
-        if (code < 0)
-            return code;
-        code = gs_lib_init1((gs_memory_t *)idmem.space_system);
-        if (code < 0)
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf1(minst->heap, "%% Init phase 1 started, instance 0x%p\n", minst);
+
+    code = ialloc_init(&idmem, minst->heap,
+                       minst->memory_clump_size, gs_have_level2());
+
+    if (code < 0)
+        goto fail_early;
+
+    code = gs_lib_init1((gs_memory_t *)idmem.space_system);
+    if (code < 0)
+        goto fail;
+    alloc_save_init(&idmem);
+    {
+        gs_memory_t *mem = (gs_memory_t *)idmem.space_system;
+        nt = names_init(minst->name_table_size, idmem.space_system);
+
+        if (nt == 0) {
+            code = gs_note_error(gs_error_VMerror);
             goto fail;
-        alloc_save_init(&idmem);
-        {
-            gs_memory_t *mem = (gs_memory_t *)idmem.space_system;
-            nt = names_init(minst->name_table_size, idmem.space_system);
-
-            if (nt == 0) {
-                code = gs_note_error(gs_error_VMerror);
-                goto fail;
-            }
-            mem->gs_lib_ctx->gs_name_table = nt;
-            code = gs_register_struct_root(mem, mem->gs_lib_ctx->name_table_root,
-                                           (void **)&mem->gs_lib_ctx->gs_name_table,
-                                           "the_gs_name_table");
-            if (code < 0)
-                goto fail;
-            mem->gs_lib_ctx->client_check_file_permission = z_check_file_permissions;
         }
-        code = obj_init(&minst->i_ctx_p, &idmem);  /* requires name_init */
-        if (code < 0)
-            goto fail;
-        minst->init_done = 1;
-        code = i_plugin_init(minst->i_ctx_p);
-        if (code < 0)
-            goto fail;
-        code = i_iodev_init(&idmem);
+        mem->gs_lib_ctx->gs_name_table = nt;
+        code = gs_register_struct_root(mem, &mem->gs_lib_ctx->name_table_root,
+                                       (void **)&mem->gs_lib_ctx->gs_name_table,
+                                       "the_gs_name_table");
         if (code < 0)
             goto fail;
+        mem->gs_lib_ctx->client_check_file_permission = z_check_file_permissions;
     }
-    return 0;
-
+    code = obj_init(&minst->i_ctx_p, &idmem);  /* requires name_init */
+    if (code < 0)
+        goto fail;
+    minst->init_done = 1;
+    code = i_plugin_init(minst->i_ctx_p);
+    if (code < 0)
+        goto fail;
+    code = i_iodev_init(&idmem);
+    if (code < 0) {
 fail:
-    names_free(nt);
-    if (minst->i_ctx_p == NULL)
-        ialloc_finit(&idmem);
+        names_free(nt);
+        if (minst->i_ctx_p == NULL)
+            ialloc_finit(&idmem);
+    }
+fail_early:
+
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf2(minst->heap, "%% Init phase 1 %s, instance 0x%p\n", code < 0 ? "failed" : "done", minst);
+
     return code;
 }
 
@@ -225,7 +243,7 @@ fail:
  */
 static int
 gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
-        int *pexit_code, ref * perror_object)
+                  int *pexit_code, ref * perror_object)
 {
     int code;
 
@@ -233,7 +251,7 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
     minst->i_ctx_p->lib_path = &minst->lib_path;
 
     code = gs_interpret(&minst->i_ctx_p, pref,
-                user_errors, pexit_code, perror_object);
+                        user_errors, pexit_code, perror_object);
     return code;
 }
 
@@ -307,18 +325,24 @@ gs_main_init2(gs_main_instance * minst)
 {
     i_ctx_t *i_ctx_p;
     int code = gs_main_init1(minst);
-    int initial_init_level = minst->init_done;
 
     if (code < 0)
         return code;
 
+    if (minst->init_done >= 2)
+        return 0;
+
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf1(minst->heap, "%% Init phase 2 started, instance 0x%p\n", minst);
+
     code = gs_main_init2aux(minst);
     if (code < 0)
-       return code;
+       goto fail;
+
     i_ctx_p = minst->i_ctx_p; /* display_set_callback or run_string may change it */
 
     /* Now process the initial saved-pages=... argument, if any as well as saved-pages-test */
-    if (initial_init_level < 2) {
+    {
        gx_device *pdev = gs_currentdevice(minst->i_ctx_p->pgs);	/* get the current device */
        gx_device_printer *ppdev = (gx_device_printer *)pdev;
 
@@ -328,30 +352,40 @@ gs_main_init2(gs_main_instance * minst)
                 minst->saved_pages_test_mode = false;  /* device doesn't support it */
             } else {
                 if ((code = gx_saved_pages_param_process(ppdev, (byte *)"begin", 5)) < 0)
-                    return code;
+                    goto fail;
                 if (code > 0)
                     if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
-                        return code;
+                        goto fail;
             }
         } else if (minst->saved_pages_initial_arg != NULL) {
             if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) <= 0) {
+                while (pdev->child)
+                    pdev = pdev->child;		/* so we get the real device name */
                 outprintf(minst->heap,
                           "   --saved-pages not supported by the '%s' device.\n",
                           pdev->dname);
-                return gs_error_Fatal;
+                code = gs_error_Fatal;
+                goto fail;
             }
             code = gx_saved_pages_param_process(ppdev, (byte *)minst->saved_pages_initial_arg,
                                                 strlen(minst->saved_pages_initial_arg));
             if (code > 0)
                 if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
-                    return code;
+                    goto fail;
         }
     }
 
-    if (gs_debug_c(':'))
-        print_resource_usage(minst, &gs_imemory, "Start");
-    gp_readline_init(&minst->readline_data, imemory_system);
-    return 0;
+fail:
+    if (gs_debug_c(gs_debug_flag_init_details))
+        dmprintf2(minst->heap, "%% Init phase 2 %s, instance 0x%p\n", code < 0 ? "failed" : "done", minst);
+
+    if (code >= 0) {
+        if (gs_debug_c(':'))
+            print_resource_usage(minst, &gs_imemory, "Start");
+        gp_readline_init(&minst->readline_data, imemory_system);
+    }
+
+    return code;
 }
 
 /* ------ Search paths ------ */
@@ -383,62 +417,112 @@ extend_path_list_container (gs_main_instance * minst, gs_file_path * pfp)
     return(0);
 }
 
+static int
+lib_path_insert_copy_of_string(gs_main_instance *minst, int pos, size_t strlen, const char *str)
+{
+    ref *paths;
+    int listlen = r_size(&minst->lib_path.list); /* The real number of entries currently in the list */
+    byte *newstr;
+    int code;
+
+    /* Extend the container if required */
+    if (listlen == r_size(&minst->lib_path.container)) {
+        code = extend_path_list_container(minst, &minst->lib_path);
+        if (code < 0) {
+            emprintf(minst->heap, "\nAdding path to search paths failed.\n");
+            return(code);
+        }
+    }
+
+    /* Copy the string */
+    newstr = gs_alloc_string(minst->heap, strlen, "lib_path_add");
+    if (newstr == NULL)
+        return gs_error_VMerror;
+    memcpy(newstr, str, strlen);
+
+    /* Shuffle up the entries */
+    paths = &minst->lib_path.container.value.refs[pos];
+    if (pos != listlen)
+        memmove(paths + 1, paths, (listlen-pos) * sizeof(*paths));
+
+    /* And insert the new string, converted to a PS thing */
+    make_const_string(paths, avm_foreign | a_readonly,
+                      strlen,
+                      newstr);
+
+    /* Update the list length */
+    r_set_size(&minst->lib_path.list, listlen+1);
+
+    return 0;
+}
+
 /* Internal routine to add a set of directories to a search list. */
 /* Returns 0 or an error code. */
 
 static int
-file_path_add(gs_main_instance * minst, gs_file_path * pfp, const char *dirs)
+lib_path_add(gs_main_instance * minst, const char *dirs)
 {
+    gs_file_path * pfp = &minst->lib_path;
     uint len = r_size(&pfp->list);
     const char *dpath = dirs;
     int code;
 
     if (dirs == 0)
         return 0;
+
     for (;;) {                  /* Find the end of the next directory name. */
         const char *npath = dpath;
 
         while (*npath != 0 && *npath != gp_file_name_list_separator)
             npath++;
         if (npath > dpath) {
-            if (len == r_size(&pfp->container)) {
-                code = extend_path_list_container(minst, pfp);
-                if (code < 0) {
-                    emprintf(minst->heap, "\nAdding path to search paths failed.\n");
-                    return(code);
-                }
-            }
-            make_const_string(&pfp->container.value.refs[len],
-                              avm_foreign | a_readonly,
-                              npath - dpath, (const byte *)dpath);
-            ++len;
+            code = lib_path_insert_copy_of_string(minst, len, npath - dpath, dpath);
+            if (code < 0)
+                return code;
+            len++;
+            /* Update length/count so we don't lose entries if a later
+             * iteration fails. */
+            r_set_size(&pfp->list, len);
         }
         if (!*npath)
             break;
         dpath = npath + 1;
     }
-    r_set_size(&pfp->list, len);
     return 0;
 }
 
+static void
+set_lib_path_length(gs_main_instance * minst, int newsize)
+{
+    gs_file_path * pfp = &minst->lib_path;
+    uint len = r_size(&pfp->list); /* Yes, list, not container */
+    uint i;
+
+    /* Free any entries that are discarded by us shortening the list */
+    for (i = newsize; i < len; i++)
+        gs_free_object(minst->heap, pfp->container.value.refs[i].value.bytes, "lib_path entry");
+    r_set_size(&pfp->list, newsize);
+}
+
 /* Add a library search path to the list. */
 int
 gs_main_add_lib_path(gs_main_instance * minst, const char *lpath)
 {
-    /* Account for the possibility that the first element */
-    /* is gp_current_directory name added by set_lib_paths. */
-    int first_is_here =
-        (r_size(&minst->lib_path.list) != 0 &&
-         minst->lib_path.container.value.refs[0].value.bytes ==
-         (const byte *)gp_current_directory_name ? 1 : 0);
+    gs_file_path * pfp = &minst->lib_path;
     int code;
 
-    r_set_size(&minst->lib_path.list, minst->lib_path.count +
-               first_is_here);
-    code = file_path_add(minst, &minst->lib_path, lpath);
-    minst->lib_path.count = r_size(&minst->lib_path.list) - first_is_here;
+    /* Throw away all the elements after the user ones. */
+    set_lib_path_length(minst, pfp->count + pfp->first_is_current);
+
+    /* Now add the new one */
+    code = lib_path_add(minst, lpath);
     if (code < 0)
         return code;
+
+    /* Update the count of user paths */
+    pfp->count = r_size(&pfp->list) - pfp->first_is_current;
+
+    /* Now add back in the others */
     return gs_main_set_lib_paths(minst);
 }
 
@@ -452,35 +536,53 @@ extern_gx_io_device_table();
 int
 gs_main_set_lib_paths(gs_main_instance * minst)
 {
-    ref *paths = minst->lib_path.container.value.refs;
-    int first_is_here =
-        (r_size(&minst->lib_path.list) != 0 &&
-         paths[0].value.bytes == (const byte *)gp_current_directory_name ? 1 : 0);
     int code = 0;
-    int count = minst->lib_path.count;
     int i, have_rom_device = 0;
 
-    if (minst->search_here_first) {
-        if (!(first_is_here ||
-              (r_size(&minst->lib_path.list) != 0 &&
-               !bytes_compare((const byte *)gp_current_directory_name,
-                              strlen(gp_current_directory_name),
-                              paths[0].value.bytes,
-                              r_size(&paths[0]))))
-            ) {
-            memmove(paths + 1, paths, count * sizeof(*paths));
-            make_const_string(paths, avm_foreign | a_readonly,
-                              strlen(gp_current_directory_name),
-                              (const byte *)gp_current_directory_name);
-        }
-    } else {
-        if (first_is_here)
-            memmove(paths, paths + 1, count * sizeof(*paths));
+    /* We are entered with a list potentially full of stuff already.
+     * In general the list is of the form:
+     *    <optionally, the current directory>
+     *    <any -I specified directories, "count" of them>
+     *    <any directories from the environment variable>
+     *    <if there is a romfs, the romfs paths>
+     *    <any directories from the 'final' variable>
+     *
+     * Unfortunately, the value read from the environment variable may not
+     * have actually been read the first time this function is called, so
+     * we can see the value change over time. Short of a complete rewrite
+     * this requires us to clear the latter part of the list and repopulate
+     * it each time we run through.
+     */
+
+    /* First step, ensure that we have the current directory as our
+     * first entry, iff we need it. */
+    if (minst->search_here_first && !minst->lib_path.first_is_current) {
+        /* We should have a "gp_current_directory" at the start, and we haven't.
+         * So insert one. */
+        code = lib_path_insert_copy_of_string(minst, 0,
+                                              strlen(gp_current_directory_name),
+                                              gp_current_directory_name);
+        if (code < 0)
+            return code;
+    } else if (!minst->search_here_first && minst->lib_path.first_is_current) {
+        /* We have a "gp_current_directory" at the start, and we shouldn't have.
+         * Remove it. */
+        int listlen = r_size(&minst->lib_path.list); /* The real number of entries currently in the list */
+        ref *paths = minst->lib_path.container.value.refs;
+
+        gs_free_object(minst->heap, paths->value.bytes, "lib_path entry");
+        --listlen;
+        memmove(paths, paths + 1, listlen * sizeof(*paths));
+        r_set_size(&minst->lib_path.list, listlen);
     }
-    r_set_size(&minst->lib_path.list,
-               count + (minst->search_here_first ? 1 : 0));
+    minst->lib_path.first_is_current = minst->search_here_first;
+
+    /* Now, the horrid bit. We throw away all the entries after the user entries */
+    set_lib_path_length(minst, minst->lib_path.count + minst->lib_path.first_is_current);
+
+    /* Now we (re)populate the end of the list. */
     if (minst->lib_path.env != 0)
-        code = file_path_add(minst, &minst->lib_path, minst->lib_path.env);
+        code = lib_path_add(minst, minst->lib_path.env);
     /* now put the %rom%lib/ device path before the gs_lib_default_path on the list */
     for (i = 0; i < gx_io_device_table_count; i++) {
         const gx_io_device *iodev = gx_io_device_table[i];
@@ -497,13 +599,13 @@ gs_main_set_lib_paths(gs_main_instance * minst)
         }
     }
     if (have_rom_device && code >= 0) {
-        code = file_path_add(minst, &minst->lib_path, "%rom%Resource/Init/");
+        code = lib_path_add(minst, "%rom%Resource/Init/");
         if (code < 0)
             return code;
-        code = file_path_add(minst, &minst->lib_path, "%rom%lib/");
+        code = lib_path_add(minst, "%rom%lib/");
     }
-    if (minst->lib_path.final != 0 && code >= 0)
-        code = file_path_add(minst, &minst->lib_path, minst->lib_path.final);
+    if (minst->lib_path.final != NULL && code >= 0)
+        code = lib_path_add(minst, minst->lib_path.final);
     return code;
 }
 
@@ -630,7 +732,7 @@ gs_main_run_string_begin(gs_main_instance * minst, int user_errors,
     make_const_string(&rstr, avm_foreign | a_readonly | a_executable,
                       strlen(setup), (const byte *)setup);
     code = gs_main_interpret(minst, &rstr, user_errors, pexit_code,
-                        perror_object);
+                             perror_object);
     return (code == gs_error_NeedInput ? 0 : code == 0 ? gs_error_Fatal : code);
 }
 /* Continue running a string with the option of suspending. */
@@ -645,8 +747,16 @@ gs_main_run_string_continue(gs_main_instance * minst, const char *str,
     make_const_string(&rstr, avm_foreign | a_readonly, length,
                       (const byte *)str);
     return gs_main_interpret(minst, &rstr, user_errors, pexit_code,
-                        perror_object);
+                             perror_object);
 }
+uint
+gs_main_get_uel_offset(gs_main_instance * minst)
+{
+    if (minst->i_ctx_p == NULL)
+        return 0;
+    return (uint)minst->i_ctx_p->uel_position;
+}
+
 /* Signal EOF when suspended. */
 int
 gs_main_run_string_end(gs_main_instance * minst, int user_errors,
@@ -656,7 +766,79 @@ gs_main_run_string_end(gs_main_instance * minst, int user_errors,
 
     make_empty_const_string(&rstr, avm_foreign | a_readonly);
     return gs_main_interpret(minst, &rstr, user_errors, pexit_code,
-                        perror_object);
+                             perror_object);
+}
+
+gs_memory_t *
+gs_main_get_device_memory(gs_main_instance * minst)
+{
+  gs_memory_t *dev_mem = NULL;
+  if (minst && minst->init_done >= 1) {
+      i_ctx_t * i_ctx_p = minst->i_ctx_p;
+      dev_mem = imemory_global->stable_memory;
+  }
+  return dev_mem;
+}
+
+int
+gs_main_set_device(gs_main_instance * minst, gx_device *pdev)
+{
+    i_ctx_t *i_ctx_p = minst->i_ctx_p;
+    ref error_object;
+    int code;
+
+    if (pdev == NULL) {
+        /* Leave job encapsulation, restore the graphics state gsaved below (so back to the nullpage device)
+           and re-enter job encapsulation.
+           We rely on the end of job encapsulation restore to the put the gstate stack back how it was when
+           we entered job encapsulation below, so the grestore will pickup the correct gstate.
+         */
+        code = gs_main_run_string(minst,
+                                 "true 0 startjob pop grestore false 0 startjob pop",
+                                 0, &code, &error_object);
+        if (code < 0) goto done;
+    }
+    else {
+        /* Leave job encapsulation, and save the graphics state (including the device: nullpage)
+           Store the page size in a dictionary, which we'll use to configure the incoming device */
+        code = gs_main_run_string(minst,
+                                  "true 0 startjob pop gsave "
+                                  /* /PageSize /GetDeviceParam .special_op will either return:
+                                   * /PageSize [ <width> <height> ] true   (if it exists) or
+                                   * false                                 (if it does not) */
+                                  "<< /PageSize /GetDeviceParam .special_op "
+                                  /* If we wanted to force a default pagesize, we'd do:
+                                   * "not { /PageSize [595 842] } if "
+                                   * but for now we'll just leave the default as it is, and do: */
+                                  "pop "
+                                  ">> "
+                                  , 0, &code, &error_object);
+        if (code < 0) goto done;
+        /* First call goes to the C directly to actually set the device. This
+         * avoids the SAFER checks. */
+        code = zsetdevice_no_safer(i_ctx_p, pdev);
+        if (code < 0) goto done;
+        code = zcurrentdevice(i_ctx_p);
+        if (code < 0) goto done;
+        code = gs_main_run_string(minst,
+                                  /* Set the device again to the same one. This determines
+                                   * whether to erase page or not, but passes the safer
+                                   * checks as the device is unchanged. */
+                                  "setdevice "
+                                  "setpagedevice "
+                                  /* GS specifics: Force the cached copy of the params to be updated. */
+                                  "currentpagedevice pop "
+                                  /* Setup the halftone */
+                                  ".setdefaultscreen "
+                                  /* Re-run the scheduled initialisation procs, in case we've just set pdfwrite */
+                                  "1183615869 internaldict /.execute_scheduled_inits get exec "
+                                  /* Re-enter job encapsulation */
+                                  "false 0 startjob pop "
+                                  , 0, &code, &error_object);
+        if (code < 0) goto done;
+    }
+done:
+    return code;
 }
 
 /* ------ Operand stack access ------ */
@@ -875,6 +1057,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     int exit_code;
     ref error_object;
     char *tempnames = NULL;
+    gs_lib_ctx_core_t *core;
 
     /* NB: need to free gs_name_table
      */
@@ -960,6 +1143,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
             gx_device *pdev = i_ctx_p->pgs->device;
             const char * dname = pdev->dname;
             gs_gc_root_t dev_root;
+            gs_gc_root_t *dev_root_ptr = &dev_root;
             /* There is a chance that, during the call to gs_main_run_string(), the interpreter may
              * decide to call the garbager - the device is in gc memory, and the only reference to it
              * (in the gstate) has been removed, thus it can be destroyed by the garbager.
@@ -967,7 +1151,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
              * Register the device as a gc 'root' so it will be implicitely marked by garbager, and
              * and thus surive until control returns here.
              */
-            if (gs_register_struct_root(pdev->memory, &dev_root, (void **)&pdev, "gs_main_finit") < 0) {
+            if (gs_register_struct_root(pdev->memory, &dev_root_ptr, (void **)&pdev, "gs_main_finit") < 0) {
                 free(tempnames);
                 return_error(gs_error_Fatal);
             }
@@ -993,7 +1177,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                     emprintf2(imemory, "UNKNOWN ERROR %d closing %s device.\n", code, dname);
                }
             }
-            gs_unregister_root(pdev->memory, &dev_root, "gs_main_finit");
+            gs_unregister_root(pdev->memory, dev_root_ptr, "gs_main_finit");
             rc_decrement(pdev, "gs_main_finit");                /* device might be freed */
             if (exit_status == 0 || exit_status == gs_error_Quit)
                 exit_status = code;
@@ -1007,6 +1191,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
         0 , &exit_code, &error_object);
     }
     gp_readline_finit(minst->readline_data);
+    gs_free_object(minst->heap, minst->saved_pages_initial_arg, "gs_main_finit");
     i_ctx_p = minst->i_ctx_p;		/* get current interp context */
     if (gs_debug_c(':')) {
         print_resource_usage(minst, &gs_imemory, "Final");
@@ -1029,15 +1214,16 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     }
 
     /* clean up redirected stdout */
-    if (minst->heap->gs_lib_ctx->fstdout2
-        && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstdout)
-        && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstderr)) {
-        fclose(minst->heap->gs_lib_ctx->fstdout2);
-        minst->heap->gs_lib_ctx->fstdout2 = (FILE *)NULL;
+    core = minst->heap->gs_lib_ctx->core;
+    if (core->fstdout2
+        && (core->fstdout2 != core->fstdout)
+        && (core->fstdout2 != core->fstderr)) {
+        fclose(core->fstdout2);
+        core->fstdout2 = (FILE *)NULL;
     }
 
-    minst->heap->gs_lib_ctx->stdout_is_redirected = 0;
-    minst->heap->gs_lib_ctx->stdout_to_stderr = 0;
+    minst->heap->gs_lib_ctx->core->stdout_is_redirected = 0;
+    minst->heap->gs_lib_ctx->core->stdout_to_stderr = 0;
     /* remove any temporary files, after ghostscript has closed files */
     if (tempnames) {
         char *p = tempnames;
@@ -1049,6 +1235,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     }
     gs_lib_finit(exit_status, code, minst->heap);
 
+    set_lib_path_length(minst, 0);
     gs_free_object(minst->heap, minst->lib_path.container.value.refs, "lib_path array");
     ialloc_finit(&dmem);
     return exit_status;
@@ -1127,3 +1314,53 @@ gs_main_dump_stack(gs_main_instance *minst, int code, ref * perror_object)
     debug_dump_stack(minst->heap, &e_stack, "Execution stack");
     debug_dump_stack(minst->heap, &d_stack, "Dictionary stack");
 }
+
+int
+gs_main_force_resolutions(gs_main_instance * minst, const float *resolutions)
+{
+    ref value;
+    int code;
+
+    if (resolutions == NULL)
+        return 0;
+
+    if (minst == NULL)
+        return gs_error_Fatal;
+
+    make_true(&value);
+    code = i_initial_enter_name(minst->i_ctx_p, "FIXEDRESOLUTION", &value);
+    if (code < 0)
+        return code;
+    make_real(&value, resolutions[0]);
+    code = i_initial_enter_name(minst->i_ctx_p, "DEVICEXRESOLUTION", &value);
+    if (code < 0)
+        return code;
+    make_real(&value, resolutions[1]);
+    return i_initial_enter_name(minst->i_ctx_p, "DEVICEYRESOLUTION", &value);
+}
+
+int
+gs_main_force_dimensions(gs_main_instance *minst, const long *dimensions)
+{
+    i_ctx_t *i_ctx_p;
+    ref value;
+    int code = 0;
+
+    if (dimensions == NULL)
+        return 0;
+    if (minst == NULL)
+        return gs_error_Fatal;
+
+    i_ctx_p = minst->i_ctx_p;
+
+    make_true(&value);
+    code = i_initial_enter_name(minst->i_ctx_p, "FIXEDMEDIA", &value);
+    if (code < 0)
+        return code;
+    make_int(&value, dimensions[0]);
+    code = i_initial_enter_name(minst->i_ctx_p, "DEVICEWIDTH", &value);
+    if (code < 0)
+        return code;
+    make_int(&value, dimensions[1]);
+    return i_initial_enter_name(minst->i_ctx_p, "DEVICEHEIGHT", &value);
+}
