@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,6 +18,9 @@
 #ifndef zchar42_INCLUDED
 #  define zchar42_INCLUDED
 
+#include "iref.h"
+#include "gxfapi.h"
+
 /* Get a Type 42 character metrics and set the cache device. */
 int zchar42_set_cache(i_ctx_t *i_ctx_p, gs_font_base *pbfont, ref *cnref,
             uint glyph_index, op_proc_t cont, op_proc_t *exec_cont);
