@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* XPS interpreter - path (vector drawing) support */
@@ -256,7 +256,7 @@ xps_parse_abbreviated_geometry(xps_context_t *ctx, char *geom)
     float smooth_x, smooth_y; /* saved cubic bezier control point for smooth curves */
     int reset_smooth;
 
-    args = xps_alloc(ctx, sizeof(char*) * (strlen(geom) + 2));
+    args = xps_alloc(ctx, sizeof(char*) * (strlen(geom) + 1));
     if (!args) {
         gs_throw(gs_error_VMerror, "out of memory: args.\n");
         return;
@@ -284,8 +284,7 @@ xps_parse_abbreviated_geometry(xps_context_t *ctx, char *geom)
         }
     }
 
-    pargs[0] = s;
-    pargs[1] = 0;
+    *pargs = s;
 
     n = pargs - args;
     i = 0;
