@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -55,13 +55,23 @@ struct xps_interp_instance_s
 #define XPS_VERSION NULL
 #define XPS_BUILD_DATE NULL
 
+static int
+xps_detect_language(const char *s, int len)
+{
+    if (len < 2)
+        return 0;
+    if (memcmp(s, "PK", 2) == 0)
+        return 100;
+    return 0;
+}
+
 static const pl_interp_characteristics_t *
-xps_imp_characteristics(const pl_interp_implementation_t *pimpl)
+xps_impl_characteristics(const pl_interp_implementation_t *pimpl)
 {
     static pl_interp_characteristics_t xps_characteristics =
     {
         "XPS",
-        "PK", /* string to recognize XPS files */
+        xps_detect_language,
         "Artifex",
         XPS_VERSION,
         XPS_BUILD_DATE,
@@ -91,7 +101,7 @@ xps_set_icc_user_params(pl_interp_implementation_t *impl, gs_gstate *pgs)
 
 /* Do per-instance interpreter allocation/init. No device is set yet */
 static int
-xps_imp_allocate_interp_instance(pl_interp_implementation_t *impl,
+xps_impl_allocate_interp_instance(pl_interp_implementation_t *impl,
                                  gs_memory_t *pmem)
 {
     xps_interp_instance_t *instance;
@@ -99,19 +109,19 @@ xps_imp_allocate_interp_instance(pl_interp_implementation_t *impl,
     gs_gstate *pgs;
 
     instance = (xps_interp_instance_t *) gs_alloc_bytes(pmem,
-            sizeof(xps_interp_instance_t), "xps_imp_allocate_interp_instance");
+            sizeof(xps_interp_instance_t), "xps_impl_allocate_interp_instance");
 
     ctx = (xps_context_t *) gs_alloc_bytes(pmem,
-            sizeof(xps_context_t), "xps_imp_allocate_interp_instance");
+            sizeof(xps_context_t), "xps_impl_allocate_interp_instance");
 
     pgs = gs_gstate_alloc(pmem);
 
     if (!instance || !ctx || !pgs)
     {
         if (instance)
-            gs_free_object(pmem, instance, "xps_imp_allocate_interp_instance");
+            gs_free_object(pmem, instance, "xps_impl_allocate_interp_instance");
         if (ctx)
-            gs_free_object(pmem, ctx, "xps_imp_allocate_interp_instance");
+            gs_free_object(pmem, ctx, "xps_impl_allocate_interp_instance");
         if (pgs)
             gs_gstate_free(pgs);
         return gs_error_VMerror;
@@ -139,7 +149,9 @@ xps_imp_allocate_interp_instance(pl_interp_implementation_t *impl,
     ctx->gray = gs_cspace_new_ICC(ctx->memory, ctx->pgs, 1);
     ctx->cmyk = gs_cspace_new_ICC(ctx->memory, ctx->pgs, 4);
     ctx->srgb = gs_cspace_new_ICC(ctx->memory, ctx->pgs, 3);
-    ctx->scrgb = gs_cspace_new_ICC(ctx->memory, ctx->pgs, 3);
+
+    /* scrgb needs special treatment */
+    ctx->scrgb = gs_cspace_new_scrgb(ctx->memory, ctx->pgs);
 
     instance->ctx = ctx;
     instance->scratch_file = NULL;
@@ -157,14 +169,33 @@ xps_imp_allocate_interp_instance(pl_interp_implementation_t *impl,
     return 0;
 }
 
+/* Prepare interp instance for the next "job" */
 static int
-xps_imp_set_device(pl_interp_implementation_t *impl, gx_device *pdevice)
+xps_impl_init_job(pl_interp_implementation_t *impl,
+                 gx_device                  *pdevice)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
     gs_c_param_list list;
     int code;
 
+    if (gs_debug_c('|'))
+        xps_zip_trace = 1;
+    if (gs_debug_c('|'))
+        xps_doc_trace = 1;
+
+    ctx->font_table = xps_hash_new(ctx);
+    ctx->colorspace_table = xps_hash_new(ctx);
+
+    ctx->start_part = NULL;
+
+    ctx->use_transparency = 1;
+    if (getenv("XPS_DISABLE_TRANSPARENCY"))
+        ctx->use_transparency = 0;
+
+    ctx->opacity_only = 0;
+    ctx->fill_rule = 0;
+
     code = gs_setdevice_no_erase(ctx->pgs, pdevice);
     if (code < 0)
         goto cleanup_setdevice;
@@ -220,7 +251,7 @@ cleanup_setdevice:
 
 /* Parse an entire random access file */
 static int
-xps_imp_process_file(pl_interp_implementation_t *impl, char *filename)
+xps_impl_process_file(pl_interp_implementation_t *impl, const char *filename)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
@@ -233,9 +264,16 @@ xps_imp_process_file(pl_interp_implementation_t *impl, char *filename)
     return code;
 }
 
+/* Do any setup for parser per-cursor */
+static int                      /* ret 0 or +ve if ok, else -ve error code */
+xps_impl_process_begin(pl_interp_implementation_t * impl)
+{
+    return 0;
+}
+
 /* Parse a cursor-full of data */
 static int
-xps_imp_process(pl_interp_implementation_t *impl, stream_cursor_read *cursor)
+xps_impl_process(pl_interp_implementation_t *impl, stream_cursor_read *cursor)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
@@ -265,11 +303,17 @@ xps_imp_process(pl_interp_implementation_t *impl, stream_cursor_read *cursor)
     return 0;
 }
 
+static int                      /* ret 0 or +ve if ok, else -ve error code */
+xps_impl_process_end(pl_interp_implementation_t * impl)
+{
+    return 0;
+}
+
 /* Skip to end of job.
  * Return 1 if done, 0 ok but EOJ not found, else negative error code.
  */
 static int
-xps_imp_flush_to_eoj(pl_interp_implementation_t *impl, stream_cursor_read *pcursor)
+xps_impl_flush_to_eoj(pl_interp_implementation_t *impl, stream_cursor_read *pcursor)
 {
     /* assume XPS cannot be pjl embedded */
     pcursor->ptr = pcursor->limit;
@@ -278,7 +322,7 @@ xps_imp_flush_to_eoj(pl_interp_implementation_t *impl, stream_cursor_read *pcurs
 
 /* Parser action for end-of-file */
 static int
-xps_imp_process_eof(pl_interp_implementation_t *impl)
+xps_impl_process_eof(pl_interp_implementation_t *impl)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
@@ -303,7 +347,7 @@ xps_imp_process_eof(pl_interp_implementation_t *impl)
 
 /* Report any errors after running a job */
 static int
-xps_imp_report_errors(pl_interp_implementation_t *impl,
+xps_impl_report_errors(pl_interp_implementation_t *impl,
         int code,           /* prev termination status */
         long file_position, /* file position of error, -1 if unknown */
         bool force_to_cout  /* force errors to cout */
@@ -312,33 +356,6 @@ xps_imp_report_errors(pl_interp_implementation_t *impl,
     return 0;
 }
 
-/* Prepare interp instance for the next "job" */
-static int
-xps_imp_init_job(pl_interp_implementation_t *impl)
-{
-    xps_interp_instance_t *instance = impl->interp_client_data;
-    xps_context_t *ctx = instance->ctx;
-
-    if (gs_debug_c('|'))
-        xps_zip_trace = 1;
-    if (gs_debug_c('|'))
-        xps_doc_trace = 1;
-
-    ctx->font_table = xps_hash_new(ctx);
-    ctx->colorspace_table = xps_hash_new(ctx);
-
-    ctx->start_part = NULL;
-
-    ctx->use_transparency = 1;
-    if (getenv("XPS_DISABLE_TRANSPARENCY"))
-        ctx->use_transparency = 0;
-
-    ctx->opacity_only = 0;
-    ctx->fill_rule = 0;
-
-    return 0;
-}
-
 static void xps_free_key_func(xps_context_t *ctx, void *ptr)
 {
     xps_free(ctx, ptr);
@@ -351,11 +368,14 @@ static void xps_free_font_func(xps_context_t *ctx, void *ptr)
 
 /* Wrap up interp instance after a "job" */
 static int
-xps_imp_dnit_job(pl_interp_implementation_t *impl)
+xps_impl_dnit_job(pl_interp_implementation_t *impl)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
-    int i;
+    int i, code;
+
+    /* return to original gstate */
+    code = gs_grestore_only(ctx->pgs); /* destroys gs_save stack */
 
     if (gs_debug_c('|'))
         xps_debug_fixdocseq(ctx);
@@ -371,33 +391,29 @@ xps_imp_dnit_job(pl_interp_implementation_t *impl)
     xps_free_fixed_pages(ctx);
     xps_free_fixed_documents(ctx);
 
-    return 0;
-}
-
-/* Remove a device from an interperter instance */
-static int
-xps_imp_remove_device(pl_interp_implementation_t *impl)
-{
-    xps_interp_instance_t *instance = impl->interp_client_data;
-    xps_context_t *ctx = instance->ctx;
-
-    /* return to original gstate */
-    return gs_grestore_only(ctx->pgs); /* destroys gs_save stack */
+    return code;
 }
 
 /* Deallocate a interpreter instance */
 static int
-xps_imp_deallocate_interp_instance(pl_interp_implementation_t *impl)
+xps_impl_deallocate_interp_instance(pl_interp_implementation_t *impl)
 {
     xps_interp_instance_t *instance = impl->interp_client_data;
     xps_context_t *ctx = instance->ctx;
     gs_memory_t *mem = ctx->memory;
 
     /* language clients don't free the font cache machinery */
+    rc_decrement_cs(ctx->gray_lin, "xps_impl_deallocate_interp_instance");
+    rc_decrement_cs(ctx->gray, "xps_impl_deallocate_interp_instance");
+    rc_decrement_cs(ctx->cmyk, "xps_impl_deallocate_interp_instance");
+    rc_decrement_cs(ctx->srgb, "xps_impl_deallocate_interp_instance");
+    rc_decrement_cs(ctx->scrgb, "xps_impl_deallocate_interp_instance");
 
-    // free gstate?
-    gs_free_object(mem, ctx, "xps_imp_deallocate_interp_instance");
-    gs_free_object(mem, instance, "xps_imp_deallocate_interp_instance");
+    gs_gstate_free(ctx->pgs);
+
+    gs_free_object(mem, ctx->fontdir, "xps_impl_deallocate_interp_instance");
+    gs_free_object(mem, ctx, "xps_impl_deallocate_interp_instance");
+    gs_free_object(mem, instance, "xps_impl_deallocate_interp_instance");
 
     return 0;
 }
@@ -405,18 +421,23 @@ xps_imp_deallocate_interp_instance(pl_interp_implementation_t *impl)
 /* Parser implementation descriptor */
 pl_interp_implementation_t xps_implementation =
 {
-    xps_imp_characteristics,
-    xps_imp_allocate_interp_instance,
-    xps_imp_set_device,
-    xps_imp_init_job,
-    xps_imp_process_file,
-    xps_imp_process,
-    xps_imp_flush_to_eoj,
-    xps_imp_process_eof,
-    xps_imp_report_errors,
-    xps_imp_dnit_job,
-    xps_imp_remove_device,
-    xps_imp_deallocate_interp_instance,
+    xps_impl_characteristics,
+    xps_impl_allocate_interp_instance,
+    NULL,                       /* get_device_memory */
+    NULL,                       /* set_param */
+    NULL,                       /* add_path */
+    NULL,                       /* post_args_init */
+    xps_impl_init_job,
+    NULL,                       /* run_prefix_commands */
+    xps_impl_process_file,
+    xps_impl_process_begin,
+    xps_impl_process,
+    xps_impl_process_end,
+    xps_impl_flush_to_eoj,
+    xps_impl_process_eof,
+    xps_impl_report_errors,
+    xps_impl_dnit_job,
+    xps_impl_deallocate_interp_instance,
     NULL,
 };
 
